@@ -6,6 +6,7 @@
 #include <QObject>
 #include <QTest>
 
+#include <Quotient/roommember.h>
 #include <Quotient/syncdata.h>
 #include <qtestcase.h>
 
@@ -50,7 +51,7 @@ void ChatBarCacheTest::empty()
     QCOMPARE(chatBarCache->replyId(), QString());
     QCOMPARE(chatBarCache->isEditing(), false);
     QCOMPARE(chatBarCache->editId(), QString());
-    QCOMPARE(chatBarCache->relationUser(), room->getUser(nullptr));
+    QCOMPARE(chatBarCache->relationUser(), room->member(QString()));
     QCOMPARE(chatBarCache->relationMessage(), QString());
     QCOMPARE(chatBarCache->attachmentPath(), QString());
 }
@@ -64,7 +65,7 @@ void ChatBarCacheTest::noRoom()
     // ChatBarCache has no parent.
 
     QTest::ignoreMessage(QtWarningMsg, "ChatBarCache created with no parent, a NeoChatRoom must be set as the parent on creation.");
-    QCOMPARE(chatBarCache->relationUser(), QVariantMap());
+    QCOMPARE(chatBarCache->relationUser(), Quotient::RoomMember());
 
     QTest::ignoreMessage(QtWarningMsg, "ChatBarCache created with no parent, a NeoChatRoom must be set as the parent on creation.");
     QCOMPARE(chatBarCache->relationMessage(), QString());
@@ -80,7 +81,7 @@ void ChatBarCacheTest::badParent()
     // ChatBarCache has no parent.
 
     QTest::ignoreMessage(QtWarningMsg, "ChatBarCache created with incorrect parent, a NeoChatRoom must be set as the parent on creation.");
-    QCOMPARE(chatBarCache->relationUser(), QVariantMap());
+    QCOMPARE(chatBarCache->relationUser(), Quotient::RoomMember());
 
     QTest::ignoreMessage(QtWarningMsg, "ChatBarCache created with incorrect parent, a NeoChatRoom must be set as the parent on creation.");
     QCOMPARE(chatBarCache->relationMessage(), QString());
@@ -98,7 +99,7 @@ void ChatBarCacheTest::reply()
     QCOMPARE(chatBarCache->replyId(), QLatin1String("$153456789:example.org"));
     QCOMPARE(chatBarCache->isEditing(), false);
     QCOMPARE(chatBarCache->editId(), QString());
-    QCOMPARE(chatBarCache->relationUser(), room->getUser(room->user(QLatin1String("@example:example.org"))));
+    QCOMPARE(chatBarCache->relationUser(), room->member(QLatin1String("@example:example.org")));
     QCOMPARE(chatBarCache->relationMessage(), QLatin1String("This is an example\ntext message"));
     QCOMPARE(chatBarCache->attachmentPath(), QString());
 }
@@ -115,7 +116,7 @@ void ChatBarCacheTest::edit()
     QCOMPARE(chatBarCache->replyId(), QString());
     QCOMPARE(chatBarCache->isEditing(), true);
     QCOMPARE(chatBarCache->editId(), QLatin1String("$153456789:example.org"));
-    QCOMPARE(chatBarCache->relationUser(), room->getUser(room->user(QLatin1String("@example:example.org"))));
+    QCOMPARE(chatBarCache->relationUser(), room->member(QLatin1String("@example:example.org")));
     QCOMPARE(chatBarCache->relationMessage(), QLatin1String("This is an example\ntext message"));
     QCOMPARE(chatBarCache->attachmentPath(), QString());
 }
@@ -132,7 +133,7 @@ void ChatBarCacheTest::attachment()
     QCOMPARE(chatBarCache->replyId(), QString());
     QCOMPARE(chatBarCache->isEditing(), false);
     QCOMPARE(chatBarCache->editId(), QString());
-    QCOMPARE(chatBarCache->relationUser(), room->getUser(nullptr));
+    QCOMPARE(chatBarCache->relationUser(), room->member(QString()));
     QCOMPARE(chatBarCache->relationMessage(), QString());
     QCOMPARE(chatBarCache->attachmentPath(), QLatin1String("some/path"));
 }
