@@ -56,6 +56,7 @@ private Q_SLOTS:
     void genericBody();
     void nullGenericBody();
     void markdownBody();
+    void markdownBodyReply();
     void subtitle();
     void nullSubtitle();
     void mediaInfo();
@@ -74,8 +75,6 @@ private Q_SLOTS:
     void nullThread();
     void location();
     void nullLocation();
-    void readMarkers();
-    void nullReadMarkers();
 };
 
 void EventHandlerTest::initTestCase()
@@ -100,28 +99,27 @@ void EventHandlerTest::nullEventId()
 void EventHandlerTest::author()
 {
     auto event = room->messageEvents().at(0).get();
-    auto author = room->user(event->senderId());
+    auto author = room->member(event->senderId());
     EventHandler eventHandler(room, event);
 
     auto eventHandlerAuthor = eventHandler.getAuthor();
 
-    QCOMPARE(eventHandlerAuthor["isLocalUser"_ls], author->id() == room->localUser()->id());
-    QCOMPARE(eventHandlerAuthor["id"_ls], author->id());
-    QCOMPARE(eventHandlerAuthor["displayName"_ls], author->displayname(room));
-    QCOMPARE(eventHandlerAuthor["avatarSource"_ls], room->avatarForMember(author));
-    QCOMPARE(eventHandlerAuthor["avatarMediaId"_ls], author->avatarMediaId(room));
-    QCOMPARE(eventHandlerAuthor["color"_ls], Utils::getUserColor(author->hueF()));
-    QCOMPARE(eventHandlerAuthor["object"_ls], QVariant::fromValue(author));
+    QCOMPARE(eventHandlerAuthor.isLocalMember(), author.id() == room->localMember().id());
+    QCOMPARE(eventHandlerAuthor.id(), author.id());
+    QCOMPARE(eventHandlerAuthor.displayName(), author.displayName());
+    QCOMPARE(eventHandlerAuthor.avatarUrl(), author.avatarUrl());
+    QCOMPARE(eventHandlerAuthor.avatarMediaId(), author.avatarMediaId());
+    QCOMPARE(eventHandlerAuthor.color(), author.color());
 }
 
 void EventHandlerTest::nullAuthor()
 {
     QTest::ignoreMessage(QtWarningMsg, "getAuthor called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.getAuthor(), QVariantMap());
+    QCOMPARE(emptyHandler.getAuthor(), RoomMember());
 
     EventHandler noEventHandler(room, nullptr);
     QTest::ignoreMessage(QtWarningMsg, "getAuthor called with m_event set to nullptr. Returning empty user.");
-    QCOMPARE(noEventHandler.getAuthor(), room->getUser(nullptr));
+    QCOMPARE(noEventHandler.getAuthor(), RoomMember());
 }
 
 void EventHandlerTest::authorDisplayName()
@@ -301,6 +299,13 @@ void EventHandlerTest::markdownBody()
     QCOMPARE(eventHandler.getMarkdownBody(), QStringLiteral("This is an example\ntext message"));
 }
 
+void EventHandlerTest::markdownBodyReply()
+{
+    EventHandler eventHandler(room, room->messageEvents().at(5).get());
+
+    QCOMPARE(eventHandler.getMarkdownBody(), QStringLiteral("reply"));
+}
+
 void EventHandlerTest::subtitle()
 {
     EventHandler eventHandler(room, room->messageEvents().at(0).get());
@@ -385,31 +390,30 @@ void EventHandlerTest::nullReplyId()
 void EventHandlerTest::replyAuthor()
 {
     auto replyEvent = room->messageEvents().at(0).get();
-    auto replyAuthor = room->user(replyEvent->senderId());
+    auto replyAuthor = room->member(replyEvent->senderId());
     EventHandler eventHandler(room, room->messageEvents().at(5).get());
 
     auto eventHandlerReplyAuthor = eventHandler.getReplyAuthor();
 
-    QCOMPARE(eventHandlerReplyAuthor["isLocalUser"_ls], replyAuthor->id() == room->localUser()->id());
-    QCOMPARE(eventHandlerReplyAuthor["id"_ls], replyAuthor->id());
-    QCOMPARE(eventHandlerReplyAuthor["displayName"_ls], replyAuthor->displayname(room));
-    QCOMPARE(eventHandlerReplyAuthor["avatarSource"_ls], room->avatarForMember(replyAuthor));
-    QCOMPARE(eventHandlerReplyAuthor["avatarMediaId"_ls], replyAuthor->avatarMediaId(room));
-    QCOMPARE(eventHandlerReplyAuthor["color"_ls], Utils::getUserColor(replyAuthor->hueF()));
-    QCOMPARE(eventHandlerReplyAuthor["object"_ls], QVariant::fromValue(replyAuthor));
+    QCOMPARE(eventHandlerReplyAuthor.isLocalMember(), replyAuthor.id() == room->localMember().id());
+    QCOMPARE(eventHandlerReplyAuthor.id(), replyAuthor.id());
+    QCOMPARE(eventHandlerReplyAuthor.displayName(), replyAuthor.displayName());
+    QCOMPARE(eventHandlerReplyAuthor.avatarUrl(), replyAuthor.avatarUrl());
+    QCOMPARE(eventHandlerReplyAuthor.avatarMediaId(), replyAuthor.avatarMediaId());
+    QCOMPARE(eventHandlerReplyAuthor.color(), replyAuthor.color());
 
     EventHandler eventHandlerNoAuthor(room, room->messageEvents().at(0).get());
-    QCOMPARE(eventHandlerNoAuthor.getReplyAuthor(), room->getUser(nullptr));
+    QCOMPARE(eventHandlerNoAuthor.getReplyAuthor(), RoomMember());
 }
 
 void EventHandlerTest::nullReplyAuthor()
 {
     QTest::ignoreMessage(QtWarningMsg, "getReplyAuthor called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.getReplyAuthor(), QVariantMap());
+    QCOMPARE(emptyHandler.getReplyAuthor(), RoomMember());
 
     EventHandler noEventHandler(room, nullptr);
     QTest::ignoreMessage(QtWarningMsg, "getReplyAuthor called with m_event set to nullptr. Returning empty user.");
-    QCOMPARE(noEventHandler.getReplyAuthor(), room->getUser(nullptr));
+    QCOMPARE(noEventHandler.getReplyAuthor(), RoomMember());
 }
 
 void EventHandlerTest::replyBody()
@@ -515,59 +519,5 @@ void EventHandlerTest::nullLocation()
     QCOMPARE(emptyHandler.getLocationAssetType(), QString());
 }
 
-void EventHandlerTest::readMarkers()
-{
-    EventHandler eventHandler(room, room->messageEvents().at(0).get());
-    QCOMPARE(eventHandler.hasReadMarkers(), true);
-
-    auto readMarkers = eventHandler.getReadMarkers();
-
-    QCOMPARE(readMarkers.size(), 1);
-    QCOMPARE(readMarkers[0].toMap()["id"_ls], QStringLiteral("@alice:matrix.org"));
-
-    QCOMPARE(eventHandler.getNumberExcessReadMarkers(), QString());
-    QCOMPARE(eventHandler.getReadMarkersString(), QStringLiteral("1 user: @alice:matrix.org"));
-
-    EventHandler eventHandler2(room, room->messageEvents().at(2).get());
-    QCOMPARE(eventHandler2.hasReadMarkers(), true);
-
-    readMarkers = eventHandler2.getReadMarkers();
-
-    QCOMPARE(readMarkers.size(), 5);
-
-    QCOMPARE(eventHandler2.getNumberExcessReadMarkers(), QStringLiteral("+ 1"));
-    // There are no guarantees on the order of the users it will be different every time so don't match the whole string.
-    QCOMPARE(eventHandler2.getReadMarkersString().startsWith(QStringLiteral("6 users:")), true);
-}
-
-void EventHandlerTest::nullReadMarkers()
-{
-    QTest::ignoreMessage(QtWarningMsg, "hasReadMarkers called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.hasReadMarkers(), false);
-
-    QTest::ignoreMessage(QtWarningMsg, "getReadMarkers called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.getReadMarkers(), QVariantList());
-
-    QTest::ignoreMessage(QtWarningMsg, "getNumberExcessReadMarkers called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.getNumberExcessReadMarkers(), QString());
-
-    QTest::ignoreMessage(QtWarningMsg, "getReadMarkersString called with m_room set to nullptr.");
-    QCOMPARE(emptyHandler.getReadMarkersString(), QString());
-
-    EventHandler noEventHandler(room, nullptr);
-
-    QTest::ignoreMessage(QtWarningMsg, "hasReadMarkers called with m_event set to nullptr.");
-    QCOMPARE(noEventHandler.hasReadMarkers(), false);
-
-    QTest::ignoreMessage(QtWarningMsg, "getReadMarkers called with m_event set to nullptr.");
-    QCOMPARE(noEventHandler.getReadMarkers(), QVariantList());
-
-    QTest::ignoreMessage(QtWarningMsg, "getNumberExcessReadMarkers called with m_event set to nullptr.");
-    QCOMPARE(noEventHandler.getNumberExcessReadMarkers(), QString());
-
-    QTest::ignoreMessage(QtWarningMsg, "getReadMarkersString called with m_event set to nullptr.");
-    QCOMPARE(noEventHandler.getReadMarkersString(), QString());
-}
-
 QTEST_MAIN(EventHandlerTest)
 #include "eventhandlertest.moc"
