@@ -19,11 +19,11 @@
 #include <Quotient/events/simplestateevents.h>
 #include <Quotient/events/stickerevent.h>
 #include <Quotient/quotient_common.h>
+#include <Quotient/roommember.h>
 
 #include "eventhandler_logging.h"
 #include "events/locationbeaconevent.h"
 #include "events/pollevent.h"
-#include "events/serveraclevent.h"
 #include "events/widgetevent.h"
 #include "linkpreviewer.h"
 #include "messagecomponenttype.h"
@@ -61,20 +61,18 @@ MessageComponentType::Type EventHandler::messageComponentType() const
     return MessageComponentType::typeForEvent(*m_event);
 }
 
-QVariantMap EventHandler::getAuthor(bool isPending) const
+Quotient::RoomMember EventHandler::getAuthor(bool isPending) const
 {
     if (m_room == nullptr) {
         qCWarning(EventHandling) << "getAuthor called with m_room set to nullptr.";
         return {};
     }
-    // If we have a room we can return an empty user by handing nullptr to m_room->getUser.
     if (m_event == nullptr) {
         qCWarning(EventHandling) << "getAuthor called with m_event set to nullptr. Returning empty user.";
-        return m_room->getUser(nullptr);
+        return {};
     }
 
-    const auto author = isPending ? m_room->localUser() : m_room->user(m_event->senderId());
-    return m_room->getUser(author);
+    return isPending ? m_room->localMember() : m_room->member(m_event->senderId());
 }
 
 QString EventHandler::getAuthorDisplayName(bool isPending) const
@@ -96,8 +94,8 @@ QString EventHandler::getAuthorDisplayName(bool isPending) const
         }
         return previousDisplayName;
     } else {
-        const auto author = isPending ? m_room->localUser() : m_room->user(m_event->senderId());
-        return m_room->htmlSafeMemberName(author->id());
+        const auto author = isPending ? m_room->localMember() : m_room->member(m_event->senderId());
+        return author.htmlSafeDisplayName();
     }
 }
 
@@ -112,8 +110,8 @@ QString EventHandler::singleLineAuthorDisplayname(bool isPending) const
         return {};
     }
 
-    const auto author = isPending ? m_room->localUser() : m_room->user(m_event->senderId());
-    auto displayName = m_room->safeMemberName(author->id());
+    const auto author = isPending ? m_room->localMember() : m_room->member(m_event->senderId());
+    auto displayName = author.displayName();
     displayName.replace(QStringLiteral("<br>\n"), QStringLiteral(" "));
     displayName.replace(QStringLiteral("<br>"), QStringLiteral(" "));
     displayName.replace(QStringLiteral("<br />\n"), QStringLiteral(" "));
@@ -220,7 +218,7 @@ bool EventHandler::isHidden()
         }
     }
 
-    if (m_room->connection()->isIgnored(m_room->user(m_event->senderId()))) {
+    if (m_room->connection()->isIgnored(m_event->senderId())) {
         return true;
     }
 
@@ -243,19 +241,21 @@ Qt::TextFormat EventHandler::messageBodyInputFormat(const Quotient::RoomMessageE
 
 QString EventHandler::rawMessageBody(const Quotient::RoomMessageEvent &event)
 {
+    QString body;
+
     if (event.hasFileContent()) {
-        auto fileCaption = event.content()->fileInfo()->originalName;
-        if (fileCaption.isEmpty()) {
-            fileCaption = event.plainBody();
-        } else if (event.content()->fileInfo()->originalName != event.plainBody()) {
-            fileCaption = event.plainBody() + " | "_ls + fileCaption;
+        // if filename is given or body is equal to filename,
+        // then body is a caption
+        QString filename = event.content()->fileInfo()->originalName;
+        QString body = event.plainBody();
+        if (filename.isEmpty() || filename == body) {
+            return QString();
         }
-        return fileCaption;
+        return body;
     }
 
-    QString body;
     if (event.hasTextContent() && event.content()) {
-        body = static_cast<const MessageEventContent::TextContent *>(event.content())->body;
+        body = static_cast<const EventContent::TextContent *>(event.content())->body;
     } else {
         body = event.plainBody();
     }
@@ -293,7 +293,10 @@ QString EventHandler::getMarkdownBody() const
     }
 
     const auto roomMessageEvent = eventCast<const RoomMessageEvent>(m_event);
-    return roomMessageEvent->plainBody();
+
+    QString plainBody = roomMessageEvent->plainBody();
+    plainBody.remove(TextRegex::removeReply);
+    return plainBody;
 }
 
 QString EventHandler::getBody(const Quotient::RoomEvent *event, Qt::TextFormat format, bool stripNewlines) const
@@ -315,15 +318,19 @@ QString EventHandler::getBody(const Quotient::RoomEvent *event, Qt::TextFormat f
         },
         [this, prettyPrint](const RoomMemberEvent &e) {
             // FIXME: Rewind to the name that was at the time of this event
-            auto subjectName = m_room->htmlSafeMemberName(e.userId());
+            auto subjectName = m_room->member(e.userId()).htmlSafeDisplayName();
             if (e.membership() == Membership::Leave) {
                 if (e.prevContent() && e.prevContent()->displayName) {
-                    subjectName = sanitized(*e.prevContent()->displayName).toHtmlEscaped();
+                    subjectName = sanitized(*e.prevContent()->displayName);
+                    if (prettyPrint) {
+                        subjectName = subjectName.toHtmlEscaped();
+                    }
                 }
             }
 
             if (prettyPrint) {
-                subjectName = QStringLiteral("<a href=\"https://matrix.to/#/%1\">%2</a>").arg(e.userId(), subjectName);
+                subjectName = QStringLiteral("<a href=\"https://matrix.to/#/%1\" style=\"color: %2\">%3</a>")
+                                  .arg(e.userId(), m_room->member(e.userId()).color().name(), subjectName);
             }
 
             // The below code assumes senderName output in AuthorRole
@@ -437,7 +444,7 @@ QString EventHandler::getBody(const Quotient::RoomEvent *event, Qt::TextFormat f
         [](const LocationBeaconEvent &e) {
             return e.contentJson()["description"_ls].toString();
         },
-        [](const ServerAclEvent &) {
+        [](const RoomServerAclEvent &) {
             return i18n("changed the server access control lists for this room");
         },
         [](const WidgetEvent &e) {
@@ -476,7 +483,7 @@ QString EventHandler::getMessageBody(const RoomMessageEvent &event, Qt::TextForm
 
     QString body;
     if (event.hasTextContent() && event.content()) {
-        body = static_cast<const MessageEventContent::TextContent *>(event.content())->body;
+        body = static_cast<const EventContent::TextContent *>(event.content())->body;
     } else {
         body = event.plainBody();
     }
@@ -606,7 +613,7 @@ QString EventHandler::getGenericBody() const
         [](const LocationBeaconEvent &) {
             return i18n("sent a live location beacon");
         },
-        [](const ServerAclEvent &) {
+        [](const RoomServerAclEvent &) {
             return i18n("changed the server access control lists for this room");
         },
         [](const WidgetEvent &e) {
@@ -655,23 +662,30 @@ QVariantMap EventHandler::getMediaInfoForEvent(const Quotient::RoomEvent *event)
     QString eventId = event->id();
 
     // Get the file info for the event.
-    const EventContent::FileInfo *fileInfo;
-    bool isSticker = false;
     if (event->is<RoomMessageEvent>()) {
         auto roomMessageEvent = eventCast<const RoomMessageEvent>(event);
         if (!roomMessageEvent->hasFileContent()) {
             return {};
         }
+
+        const EventContent::FileInfo *fileInfo;
         fileInfo = roomMessageEvent->content()->fileInfo();
+        QVariantMap mediaInfo = getMediaInfoFromFileInfo(fileInfo, eventId, false, false);
+        // if filename isn't specifically given, it is in body
+        // https://spec.matrix.org/latest/client-server-api/#mfile
+        mediaInfo["filename"_ls] = (fileInfo->originalName.isEmpty()) ? roomMessageEvent->plainBody() : fileInfo->originalName;
+
+        return mediaInfo;
     } else if (event->is<StickerEvent>()) {
+        const EventContent::FileInfo *fileInfo;
+
         auto stickerEvent = eventCast<const StickerEvent>(event);
         fileInfo = &stickerEvent->image();
-        isSticker = true;
+
+        return getMediaInfoFromFileInfo(fileInfo, eventId, false, true);
     } else {
         return {};
     }
-
-    return getMediaInfoFromFileInfo(fileInfo, eventId, false, isSticker);
 }
 
 QVariantMap EventHandler::getMediaInfoFromFileInfo(const EventContent::FileInfo *fileInfo, const QString &eventId, bool isThumbnail, bool isSticker) const
@@ -797,25 +811,21 @@ MessageComponentType::Type EventHandler::replyMessageComponentType() const
     return MessageComponentType::typeForEvent(*replyEvent);
 }
 
-QVariantMap EventHandler::getReplyAuthor() const
+Quotient::RoomMember EventHandler::getReplyAuthor() const
 {
     if (m_room == nullptr) {
         qCWarning(EventHandling) << "getReplyAuthor called with m_room set to nullptr.";
         return {};
     }
-    // If we have a room we can return an empty user by handing nullptr to m_room->getUser.
     if (m_event == nullptr) {
         qCWarning(EventHandling) << "getReplyAuthor called with m_event set to nullptr. Returning empty user.";
-        return m_room->getUser(nullptr);
+        return {};
     }
 
-    auto replyPtr = m_room->getReplyForEvent(*m_event);
-
-    if (replyPtr) {
-        auto replyUser = m_room->user(replyPtr->senderId());
-        return m_room->getUser(replyUser);
+    if (auto replyPtr = m_room->getReplyForEvent(*m_event)) {
+        return m_room->member(replyPtr->senderId());
     } else {
-        return m_room->getUser(nullptr);
+        return m_room->member(QString());
     }
 }
 
@@ -951,101 +961,4 @@ QString EventHandler::getLocationAssetType() const
     return assetType;
 }
 
-bool EventHandler::hasReadMarkers() const
-{
-    if (m_room == nullptr) {
-        qCWarning(EventHandling) << "hasReadMarkers called with m_room set to nullptr.";
-        return false;
-    }
-    if (m_event == nullptr) {
-        qCWarning(EventHandling) << "hasReadMarkers called with m_event set to nullptr.";
-        return false;
-    }
-
-    auto userIds = m_room->userIdsAtEvent(m_event->id());
-    userIds.remove(m_room->localUser()->id());
-    return userIds.size() > 0;
-}
-
-QVariantList EventHandler::getReadMarkers(int maxMarkers) const
-{
-    if (m_room == nullptr) {
-        qCWarning(EventHandling) << "getReadMarkers called with m_room set to nullptr.";
-        return {};
-    }
-    if (m_event == nullptr) {
-        qCWarning(EventHandling) << "getReadMarkers called with m_event set to nullptr.";
-        return {};
-    }
-
-    auto userIds_temp = m_room->userIdsAtEvent(m_event->id());
-    userIds_temp.remove(m_room->localUser()->id());
-
-    auto userIds = userIds_temp.values();
-    if (userIds.count() > maxMarkers) {
-        userIds = userIds.mid(0, maxMarkers);
-    }
-
-    QVariantList users;
-    users.reserve(userIds.size());
-    for (const auto &userId : userIds) {
-        auto user = m_room->user(userId);
-        users += m_room->getUser(user);
-    }
-
-    return users;
-}
-
-QString EventHandler::getNumberExcessReadMarkers(int maxMarkers) const
-{
-    if (m_room == nullptr) {
-        qCWarning(EventHandling) << "getNumberExcessReadMarkers called with m_room set to nullptr.";
-        return {};
-    }
-    if (m_event == nullptr) {
-        qCWarning(EventHandling) << "getNumberExcessReadMarkers called with m_event set to nullptr.";
-        return {};
-    }
-
-    auto userIds = m_room->userIdsAtEvent(m_event->id());
-    userIds.remove(m_room->localUser()->id());
-
-    if (userIds.count() > maxMarkers) {
-        return QStringLiteral("+ ") + QString::number(userIds.count() - maxMarkers);
-    } else {
-        return QString();
-    }
-}
-
-QString EventHandler::getReadMarkersString() const
-{
-    if (m_room == nullptr) {
-        qCWarning(EventHandling) << "getReadMarkersString called with m_room set to nullptr.";
-        return {};
-    }
-    if (m_event == nullptr) {
-        qCWarning(EventHandling) << "getReadMarkersString called with m_event set to nullptr.";
-        return {};
-    }
-
-    auto userIds = m_room->userIdsAtEvent(m_event->id());
-    userIds.remove(m_room->localUser()->id());
-
-    /**
-     * The string ends up in the form
-     * "x users: user1DisplayName, user2DisplayName, etc."
-     */
-    QString readMarkersString = i18np("1 user: ", "%1 users: ", userIds.size());
-    for (const auto &userId : userIds) {
-        auto user = m_room->user(userId);
-        auto displayName = user->displayname(m_room);
-        if (displayName.isEmpty()) {
-            displayName = userId;
-        }
-        readMarkersString += displayName + i18nc("list separator", ", ");
-    }
-    readMarkersString.chop(2);
-    return readMarkersString;
-}
-
 #include "moc_eventhandler.cpp"
