@@ -11,7 +11,7 @@
 #include <Quotient/events/redactionevent.h>
 #include <Quotient/events/roommessageevent.h>
 #include <Quotient/events/stickerevent.h>
-#include <Quotient/user.h>
+#include <Quotient/roommember.h>
 
 #include <QDebug>
 #include <QGuiApplication>
@@ -26,6 +26,7 @@
 #include "messagecontentmodel.h"
 #include "models/messagefiltermodel.h"
 #include "models/reactionmodel.h"
+#include "readmarkermodel.h"
 #include "texthandler.h"
 
 using namespace Quotient;
@@ -36,7 +37,6 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[DelegateTypeRole] = "delegateType";
     roles[EventIdRole] = "eventId";
     roles[TimeRole] = "time";
-    roles[TimeStringRole] = "timeString";
     roles[SectionRole] = "section";
     roles[AuthorRole] = "author";
     roles[HighlightRole] = "isHighlighted";
@@ -46,8 +46,6 @@ QHash<int, QByteArray> MessageEventModel::roleNames() const
     roles[ThreadRootRole] = "threadRoot";
     roles[ShowSectionRole] = "showSection";
     roles[ReadMarkersRole] = "readMarkers";
-    roles[ExcessReadMarkersRole] = "excessReadMarkers";
-    roles[ReadMarkersStringRole] = "readMarkersString";
     roles[ShowReadMarkersRole] = "showReadMarkers";
     roles[ReactionRole] = "reaction";
     roles[ShowReactionsRole] = "showReactions";
@@ -84,12 +82,17 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         return;
     }
 
-    beginResetModel();
     if (m_currentRoom) {
+        // HACK: Reset the model to a null room first to make sure QML dismantles
+        // last room's objects before the room is actually changed
+        beginResetModel();
+        m_readMarkerModels.clear();
         m_currentRoom->disconnect(this);
-        m_reactionModels.clear();
+        m_currentRoom = nullptr;
+        endResetModel();
     }
 
+    beginResetModel();
     m_currentRoom = room;
     Q_EMIT roomChanged();
     if (room) {
@@ -97,9 +100,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         room->setDisplayed();
 
         for (auto event = m_currentRoom->messageEvents().begin(); event != m_currentRoom->messageEvents().end(); ++event) {
-            if (const auto &roomMessageEvent = &*event->viewAs<RoomMessageEvent>()) {
-                createEventObjects(roomMessageEvent);
-            }
+            createEventObjects(&*event->viewAs<RoomEvent>());
             if (event->event()->is<PollStartEvent>()) {
                 m_currentRoom->createPollHandler(eventCast<const PollStartEvent>(event->event()));
             }
@@ -112,11 +113,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
 
         connect(m_currentRoom, &Room::aboutToAddNewMessages, this, [this](RoomEventsRange events) {
             for (auto &&event : events) {
-                const RoomMessageEvent *message = dynamic_cast<RoomMessageEvent *>(event.get());
-
-                if (message != nullptr) {
-                    createEventObjects(message);
-                }
+                createEventObjects(event.get());
                 if (event->is<PollStartEvent>()) {
                     m_currentRoom->createPollHandler(eventCast<const PollStartEvent>(event.get()));
                 }
@@ -126,9 +123,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
         });
         connect(m_currentRoom, &Room::aboutToAddHistoricalMessages, this, [this](RoomEventsRange events) {
             for (auto &event : events) {
-                if (const auto &roomMessageEvent = dynamic_cast<RoomMessageEvent *>(event.get())) {
-                    createEventObjects(roomMessageEvent);
-                }
+                createEventObjects(event.get());
                 if (event->is<PollStartEvent>()) {
                     m_currentRoom->createPollHandler(eventCast<const PollStartEvent>(event.get()));
                 }
@@ -149,7 +144,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             if (biggest < m_currentRoom->maxTimelineIndex()) {
                 auto rowBelowInserted = m_currentRoom->maxTimelineIndex() - biggest + timelineBaseIndex() - 1;
-                refreshEventRoles(rowBelowInserted, {MessageFilterModel::ShowAuthorRole});
+                refreshEventRoles(rowBelowInserted, {ContentModelRole});
             }
             for (auto i = m_currentRoom->maxTimelineIndex() - biggest; i <= m_currentRoom->maxTimelineIndex() - lowest; ++i) {
                 refreshLastUserEvents(i);
@@ -176,13 +171,13 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
                 endMoveRows();
                 movingEvent = false;
             }
-            refreshRow(timelineBaseIndex()); // Refresh the looks
+            fullEventRefresh(timelineBaseIndex());
             refreshLastUserEvents(0);
             if (timelineBaseIndex() > 0) { // Refresh below, see #312
-                refreshEventRoles(timelineBaseIndex() - 1, {MessageFilterModel::ShowAuthorRole});
+                refreshEventRoles(timelineBaseIndex() - 1, {ContentModelRole});
             }
         });
-        connect(m_currentRoom, &Room::pendingEventChanged, this, &MessageEventModel::refreshRow);
+        connect(m_currentRoom, &Room::pendingEventChanged, this, &MessageEventModel::fullEventRefresh);
         connect(m_currentRoom, &Room::pendingEventAboutToDiscard, this, [this](int i) {
             beginRemoveRows({}, i, i);
         });
@@ -192,10 +187,7 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             moveReadMarker(toEventId);
         });
         connect(m_currentRoom, &Room::replacedEvent, this, [this](const RoomEvent *newEvent) {
-            const RoomMessageEvent *message = eventCast<const RoomMessageEvent>(newEvent);
-            if (message != nullptr) {
-                createEventObjects(message);
-            }
+            createEventObjects(newEvent);
         });
         connect(m_currentRoom, &Room::updatedEvent, this, [this](const QString &eventId) {
             if (eventId.isEmpty()) { // How did we get here?
@@ -203,26 +195,43 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
             }
             const auto eventIt = m_currentRoom->findInTimeline(eventId);
             if (eventIt != m_currentRoom->historyEdge()) {
-                if (const auto &event = dynamic_cast<const RoomMessageEvent *>(&**eventIt)) {
-                    createEventObjects(event);
-                }
+                createEventObjects(eventIt->event());
                 if (eventIt->event()->is<PollStartEvent>()) {
                     m_currentRoom->createPollHandler(eventCast<const PollStartEvent>(eventIt->event()));
                 }
             }
             refreshEventRoles(eventId, {Qt::DisplayRole});
         });
-        connect(m_currentRoom, &Room::changed, this, [this]() {
-            for (auto it = m_currentRoom->messageEvents().rbegin(); it != m_currentRoom->messageEvents().rend(); ++it) {
-                auto event = it->event();
-                refreshEventRoles(event->id(), {ReadMarkersRole, ReadMarkersStringRole, ExcessReadMarkersRole});
+        connect(m_currentRoom, &Room::changed, this, [this](Room::Changes changes) {
+            if (changes.testFlag(Quotient::Room::Change::Other)) {
+                // this is slow
+                for (auto it = m_currentRoom->messageEvents().rbegin(); it != m_currentRoom->messageEvents().rend(); ++it) {
+                    createEventObjects(it->event());
+                }
             }
         });
         connect(m_currentRoom->connection(), &Connection::ignoredUsersListChanged, this, [this] {
             beginResetModel();
             endResetModel();
         });
-        qCDebug(MessageEvent) << "Connected to room" << room->id() << "as" << room->localUser()->id();
+        connect(m_currentRoom, &Room::memberNameUpdated, this, [this](RoomMember member) {
+            for (auto it = m_currentRoom->messageEvents().rbegin(); it != m_currentRoom->messageEvents().rend(); ++it) {
+                auto event = it->event();
+                if (event->senderId() == member.id()) {
+                    refreshEventRoles(event->id(), {AuthorRole});
+                }
+            }
+        });
+        connect(m_currentRoom, &Room::memberAvatarUpdated, this, [this](RoomMember member) {
+            for (auto it = m_currentRoom->messageEvents().rbegin(); it != m_currentRoom->messageEvents().rend(); ++it) {
+                auto event = it->event();
+                if (event->senderId() == member.id()) {
+                    refreshEventRoles(event->id(), {AuthorRole});
+                }
+            }
+        });
+
+        qCDebug(MessageEvent) << "Connected to room" << room->id() << "as" << room->localMember().id();
     } else {
         lastReadEventId.clear();
     }
@@ -235,14 +244,15 @@ void MessageEventModel::setRoom(NeoChatRoom *room)
     }
 }
 
-int MessageEventModel::refreshEvent(const QString &eventId)
+void MessageEventModel::fullEventRefresh(int row)
 {
-    return refreshEventRoles(eventId);
-}
-
-void MessageEventModel::refreshRow(int row)
-{
-    refreshEventRoles(row);
+    auto roles = roleNames().keys();
+    // The author of an event never changes so should only be updated when a member
+    // changed signal is emitted.
+    // This also avoids any race conditions where a member is updating and this refresh
+    // tries to access a member event that has already been deleted.
+    roles.removeAll(AuthorRole);
+    refreshEventRoles(row, roles);
 }
 
 int MessageEventModel::timelineBaseIndex() const
@@ -335,11 +345,11 @@ QDateTime MessageEventModel::makeMessageTimestamp(const Quotient::Room::rev_iter
     using Quotient::TimelineItem;
     auto rit = std::find_if(baseIt, timeline.rend(), hasValidTimestamp);
     if (rit != timeline.rend()) {
-        return {rit->event()->originTimestamp().date(), {0, 0}, Qt::LocalTime};
+        return {rit->event()->originTimestamp().date(), {0, 0}, QTimeZone::LocalTime};
     };
     auto it = std::find_if(baseIt.base(), timeline.end(), hasValidTimestamp);
     if (it != timeline.end()) {
-        return {it->event()->originTimestamp().date(), {0, 0}, Qt::LocalTime};
+        return {it->event()->originTimestamp().date(), {0, 0}, QTimeZone::LocalTime};
     };
 
     // What kind of room is that?..
@@ -358,8 +368,7 @@ void MessageEventModel::refreshLastUserEvents(int baseTimelineRow)
     const auto limit = timelineBottom + std::min(baseTimelineRow + 10, m_currentRoom->timelineSize());
     for (auto it = timelineBottom + std::max(baseTimelineRow - 10, 0); it != limit; ++it) {
         if ((*it)->senderId() == lastSender) {
-            auto idx = index(it - timelineBottom);
-            Q_EMIT dataChanged(idx, idx);
+            fullEventRefresh(it - timelineBottom);
         }
     }
 }
@@ -440,12 +449,12 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == ContentModelRole) {
-        if (!evt.isStateEvent()) {
-            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&evt, m_currentRoom));
+        if (!evt.isStateEvent() && !evt.id().isEmpty()) {
+            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_currentRoom, &evt));
         }
         if (evt.isStateEvent()) {
             if (evt.matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
-                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&evt, m_currentRoom));
+                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_currentRoom, &evt));
             }
         }
         return {};
@@ -460,7 +469,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == AuthorRole) {
-        return eventHandler.getAuthor(isPending);
+        return QVariant::fromValue(eventHandler.getAuthor(isPending));
     }
 
     if (role == HighlightRole) {
@@ -491,11 +500,11 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     if (role == ProgressInfoRole) {
         if (auto e = eventCast<const RoomMessageEvent>(&evt)) {
             if (e->hasFileContent()) {
-                return QVariant::fromValue(m_currentRoom->fileTransferInfo(e->id()));
+                return QVariant::fromValue(m_currentRoom->cachedFileTransferInfo(&evt));
             }
         }
-        if (auto e = eventCast<const StickerEvent>(&evt)) {
-            return QVariant::fromValue(m_currentRoom->fileTransferInfo(e->id()));
+        if (eventCast<const StickerEvent>(&evt)) {
+            return QVariant::fromValue(m_currentRoom->cachedFileTransferInfo(&evt));
         }
     }
 
@@ -504,11 +513,6 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
         return eventHandler.getTime(isPending, lastUpdated);
     }
 
-    if (role == TimeStringRole) {
-        auto lastUpdated = isPending ? pendingIt->lastUpdated() : QDateTime();
-        return eventHandler.getTimeString(false, QLocale::ShortFormat, isPending, lastUpdated);
-    }
-
     if (role == SectionRole) {
         auto lastUpdated = isPending ? pendingIt->lastUpdated() : QDateTime();
         return eventHandler.getTimeString(true, QLocale::ShortFormat, isPending, lastUpdated);
@@ -539,19 +543,15 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == ReadMarkersRole) {
-        return eventHandler.getReadMarkers();
-    }
-
-    if (role == ExcessReadMarkersRole) {
-        return eventHandler.getNumberExcessReadMarkers();
-    }
-
-    if (role == ReadMarkersStringRole) {
-        return eventHandler.getReadMarkersString();
+        if (m_readMarkerModels.contains(evt.id())) {
+            return QVariant::fromValue<ReadMarkerModel *>(m_readMarkerModels[evt.id()].get());
+        } else {
+            return QVariantList();
+        }
     }
 
     if (role == ShowReadMarkersRole) {
-        return eventHandler.hasReadMarkers();
+        return m_readMarkerModels.contains(evt.id());
     }
 
     if (role == ReactionRole) {
@@ -592,7 +592,7 @@ QVariant MessageEventModel::data(const QModelIndex &idx, int role) const
     }
 
     if (role == IsEditableRole) {
-        return eventHandler.messageComponentType() == MessageComponentType::Text && evt.senderId() == m_currentRoom->localUser()->id();
+        return eventHandler.messageComponentType() == MessageComponentType::Text && evt.senderId() == m_currentRoom->localMember().id();
     }
 
     return {};
@@ -612,30 +612,61 @@ int MessageEventModel::eventIdToRow(const QString &eventID) const
     return it - m_currentRoom->messageEvents().rbegin() + timelineBaseIndex();
 }
 
-void MessageEventModel::createEventObjects(const Quotient::RoomMessageEvent *event)
+void MessageEventModel::createEventObjects(const Quotient::RoomEvent *event)
 {
+    if (event == nullptr) {
+        return;
+    }
+
     auto eventId = event->id();
 
-    // ReactionModel handles updates to add and remove reactions, we only need to
+    // ReadMarkerModel handles updates to add and remove markers, we only need to
     // handle adding and removing whole models here.
-    if (m_reactionModels.contains(eventId)) {
+    if (m_readMarkerModels.contains(eventId)) {
         // If a model already exists but now has no reactions remove it
-        if (m_reactionModels[eventId]->rowCount() <= 0) {
-            m_reactionModels.remove(eventId);
+        if (m_readMarkerModels[eventId]->rowCount() <= 0) {
+            m_readMarkerModels.remove(eventId);
             if (!resetting) {
-                refreshEventRoles(eventId, {ReactionRole, ShowReactionsRole});
+                refreshEventRoles(eventId, {ReadMarkersRole, ShowReadMarkersRole});
             }
         }
     } else {
-        if (m_currentRoom->relatedEvents(*event, Quotient::EventRelation::AnnotationType).count() > 0) {
+        auto memberIds = m_currentRoom->userIdsAtEvent(eventId);
+        memberIds.remove(m_currentRoom->localMember().id());
+        if (memberIds.size() > 0) {
             // If a model doesn't exist and there are reactions add it.
-            auto reactionModel = QSharedPointer<ReactionModel>(new ReactionModel(event, m_currentRoom));
-            if (reactionModel->rowCount() > 0) {
-                m_reactionModels[eventId] = reactionModel;
+            auto newModel = QSharedPointer<ReadMarkerModel>(new ReadMarkerModel(eventId, m_currentRoom));
+            if (newModel->rowCount() > 0) {
+                m_readMarkerModels[eventId] = newModel;
+                if (!resetting) {
+                    refreshEventRoles(eventId, {ReadMarkersRole, ShowReadMarkersRole});
+                }
+            }
+        }
+    }
+
+    if (const auto roomEvent = eventCast<const RoomMessageEvent>(event)) {
+        // ReactionModel handles updates to add and remove reactions, we only need to
+        // handle adding and removing whole models here.
+        if (m_reactionModels.contains(eventId)) {
+            // If a model already exists but now has no reactions remove it
+            if (m_reactionModels[eventId]->rowCount() <= 0) {
+                m_reactionModels.remove(eventId);
                 if (!resetting) {
                     refreshEventRoles(eventId, {ReactionRole, ShowReactionsRole});
                 }
             }
+        } else {
+            if (m_currentRoom->relatedEvents(*event, Quotient::EventRelation::AnnotationType).count() > 0) {
+                // If a model doesn't exist and there are reactions add it.
+                auto reactionModel = QSharedPointer<ReactionModel>(new ReactionModel(roomEvent, m_currentRoom));
+                if (reactionModel->rowCount() > 0) {
+                    m_reactionModels[eventId] = reactionModel;
+                    if (!resetting) {
+                        refreshEventRoles(eventId, {ReactionRole, ShowReactionsRole});
+                    }
+                }
+            }
         }
     }
 }
