@@ -10,6 +10,7 @@
 #include "linkpreviewer.h"
 #include "neochatroom.h"
 #include "pollhandler.h"
+#include "readmarkermodel.h"
 
 class ReactionModel;
 
@@ -42,7 +43,6 @@ public:
         DelegateTypeRole = Qt::UserRole + 1, /**< The delegate type of the message. */
         EventIdRole, /**< The matrix event ID of the event. */
         TimeRole, /**< The timestamp for when the event was sent (as a QDateTime). */
-        TimeStringRole, /**< The timestamp for when the event was sent as a string (in QLocale::ShortFormat). */
         SectionRole, /**< The date of the event as a string. */
         AuthorRole, /**< The author of the event. */
         HighlightRole, /**< Whether the event should be highlighted. */
@@ -59,8 +59,6 @@ public:
         ShowSectionRole, /**< Whether the section header should be shown. */
 
         ReadMarkersRole, /**< The first 5 other users at the event for read marker tracking. */
-        ExcessReadMarkersRole, /**< The number of other users at the event after the first 5. */
-        ReadMarkersStringRole, /**< String with the display name and mxID of the users at the event. */
         ShowReadMarkersRole, /**< Whether there are any other user read markers to be shown. */
         ReactionRole, /**< List model for this event. */
         ShowReactionsRole, /**< Whether there are any reactions to be shown. */
@@ -108,10 +106,6 @@ public:
 protected:
     bool event(QEvent *event) override;
 
-private Q_SLOTS:
-    int refreshEvent(const QString &eventId);
-    void refreshRow(int row);
-
 private:
     QPointer<NeoChatRoom> m_currentRoom = nullptr;
     QString lastReadEventId;
@@ -121,6 +115,7 @@ private:
     bool movingEvent = false;
     KFormat m_format;
 
+    QMap<QString, QSharedPointer<ReadMarkerModel>> m_readMarkerModels;
     QMap<QString, QSharedPointer<ReactionModel>> m_reactionModels;
 
     [[nodiscard]] int timelineBaseIndex() const;
@@ -129,12 +124,13 @@ private:
     bool canFetchMore(const QModelIndex &parent) const override;
     void fetchMore(const QModelIndex &parent) override;
 
+    void fullEventRefresh(int row);
     void refreshLastUserEvents(int baseTimelineRow);
     void refreshEventRoles(int row, const QList<int> &roles = {});
     int refreshEventRoles(const QString &eventId, const QList<int> &roles = {});
     void moveReadMarker(const QString &toEventId);
 
-    void createEventObjects(const Quotient::RoomMessageEvent *event);
+    void createEventObjects(const Quotient::RoomEvent *event);
     // Hack to ensure that we don't call endInsertRows when we haven't called beginInsertRows
     bool m_initialized = false;
 
