@@ -44,7 +44,7 @@ void SearchModel::search()
     }
 
     RoomEventFilter filter;
-    filter.unreadThreadNotifications = none;
+    filter.unreadThreadNotifications = std::nullopt;
     filter.lazyLoadMembers = true;
     filter.includeRedundantMembers = false;
     filter.notRooms = QStringList();
@@ -58,7 +58,7 @@ void SearchModel::search()
         .orderBy = "recent"_ls,
         .eventContext = SearchJob::IncludeEventContext{3, 3, true},
         .includeState = false,
-        .groupings = none,
+        .groupings = std::nullopt,
 
     };
 
@@ -82,10 +82,8 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
     EventHandler eventHandler(m_room, &event);
 
     switch (role) {
-    case ShowAuthorRole:
-        return true;
     case AuthorRole:
-        return eventHandler.getAuthor();
+        return QVariant::fromValue(eventHandler.getAuthor());
     case ShowSectionRole:
         if (row == 0) {
             return true;
@@ -93,10 +91,6 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
         return event.originTimestamp().date() != m_result->results[row - 1].result->originTimestamp().date();
     case SectionRole:
         return eventHandler.getTimeString(true);
-    case TimeRole:
-        return eventHandler.getTime();
-    case TimeStringRole:
-        return eventHandler.getTimeString(false);
     case ShowReactionsRole:
         return false;
     case ShowReadMarkersRole:
@@ -113,15 +107,18 @@ QVariant SearchModel::data(const QModelIndex &index, int role) const
         return eventHandler.threadRoot();
     case ContentModelRole: {
         if (!event.isStateEvent()) {
-            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&event, m_room));
+            return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_room, &event));
         }
         if (event.isStateEvent()) {
             if (event.matrixType() == QStringLiteral("org.matrix.msc3672.beacon_info")) {
-                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(&event, m_room));
+                return QVariant::fromValue<MessageContentModel *>(new MessageContentModel(m_room, &event));
             }
         }
         return {};
     }
+    case IsEditableRole: {
+        return false;
+    }
     }
     return DelegateType::Message;
 }
@@ -142,9 +139,6 @@ QHash<int, QByteArray> SearchModel::roleNames() const
         {AuthorRole, "author"},
         {ShowSectionRole, "showSection"},
         {SectionRole, "section"},
-        {TimeRole, "time"},
-        {TimeStringRole, "timeString"},
-        {ShowAuthorRole, "showAuthor"},
         {EventIdRole, "eventId"},
         {ExcessReadMarkersRole, "excessReadMarkers"},
         {HighlightRole, "isHighlighted"},
@@ -158,6 +152,7 @@ QHash<int, QByteArray> SearchModel::roleNames() const
         {IsThreadedRole, "isThreaded"},
         {ThreadRootRole, "threadRoot"},
         {ContentModelRole, "contentModel"},
+        {IsEditableRole, "isEditable"},
     };
 }
 
