@@ -52,12 +52,9 @@ public:
      */
     enum Roles {
         DelegateTypeRole = Qt::DisplayRole + 1,
-        ShowAuthorRole,
         AuthorRole,
         ShowSectionRole,
         SectionRole,
-        TimeRole,
-        TimeStringRole,
         EventIdRole,
         ExcessReadMarkersRole,
         HighlightRole,
@@ -71,6 +68,7 @@ public:
         IsThreadedRole,
         ThreadRootRole,
         ContentModelRole,
+        IsEditableRole,
     };
     Q_ENUM(Roles)
     explicit SearchModel(QObject *parent = nullptr);
@@ -122,7 +120,7 @@ private:
 
     QString m_searchText;
     QPointer<NeoChatRoom> m_room;
-    Quotient::Omittable<Quotient::SearchJob::ResultRoomEvents> m_result = Quotient::none;
+    std::optional<Quotient::SearchJob::ResultRoomEvents> m_result = std::nullopt;
     Quotient::SearchJob *m_job = nullptr;
     bool m_searching = false;
 };
