@@ -11,9 +11,9 @@
 
 #include <Quotient/jobs/basejob.h>
 #include <Quotient/quotient_common.h>
-#include <Quotient/user.h>
 #include <qcoro/qcorosignal.h>
 
+#include <Quotient/avatar.h>
 #include <Quotient/connection.h>
 #include <Quotient/csapi/account-data.h>
 #include <Quotient/csapi/directory.h>
@@ -36,10 +36,12 @@
 
 #include "chatbarcache.h"
 #include "clipboard.h"
+#include "controller.h"
 #include "eventhandler.h"
 #include "events/joinrulesevent.h"
 #include "events/pollevent.h"
 #include "filetransferpseudojob.h"
+#include "jobs/neochatgetcommonroomsjob.h"
 #include "neochatconfig.h"
 #include "notificationsmanager.h"
 #include "roomlastmessageprovider.h"
@@ -68,8 +70,30 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
         setFileUploadingProgress(0);
         setHasFileUploading(false);
     });
+    connect(this, &Room::fileTransferCompleted, this, [this](QString eventId) {
+        const auto evtIt = findInTimeline(eventId);
+        if (evtIt != messageEvents().rend()) {
+            const auto m_event = evtIt->viewAs<RoomEvent>();
+            QString mxcUrl;
+            if (auto event = eventCast<const Quotient::RoomMessageEvent>(m_event)) {
+                if (event->hasFileContent()) {
+                    mxcUrl = event->content()->fileInfo()->url().toString();
+                }
+            } else if (auto event = eventCast<const Quotient::StickerEvent>(m_event)) {
+                mxcUrl = event->image().fileInfo()->url().toString();
+            }
+            if (mxcUrl.isEmpty()) {
+                return;
+            }
+            auto localPath = this->fileTransferInfo(eventId).localPath.toLocalFile();
+            auto config = KSharedConfig::openStateConfig(QStringLiteral("neochatdownloads"))->group(QStringLiteral("downloads"));
+            config.writePathEntry(mxcUrl.mid(6), localPath);
+        }
+    });
 
-    connect(this, &Room::aboutToAddHistoricalMessages, this, &NeoChatRoom::readMarkerLoadedChanged);
+    connect(this, &Room::addedMessages, this, &NeoChatRoom::readMarkerLoadedChanged);
+    connect(this, &Room::aboutToAddHistoricalMessages, this, &NeoChatRoom::cleanupExtraEventRange);
+    connect(this, &Room::aboutToAddNewMessages, this, &NeoChatRoom::cleanupExtraEventRange);
 
     const auto &roomLastMessageProvider = RoomLastMessageProvider::self();
 
@@ -94,23 +118,52 @@ NeoChatRoom::NeoChatRoom(Connection *connection, QString roomId, JoinState joinS
     });
     connect(this, &Room::displaynameChanged, this, &NeoChatRoom::displayNameChanged);
 
-    connectSingleShot(this, &Room::baseStateLoaded, this, [this]() {
-        updatePushNotificationState(QStringLiteral("m.push_rules"));
+    connect(
+        this,
+        &Room::baseStateLoaded,
+        this,
+        [this]() {
+            updatePushNotificationState(QStringLiteral("m.push_rules"));
 
-        Q_EMIT canEncryptRoomChanged();
-        if (this->joinState() != JoinState::Invite) {
-            return;
-        }
-        auto roomMemberEvent = currentState().get<RoomMemberEvent>(localUser()->id());
-        QImage avatar_image;
-        if (roomMemberEvent && !user(roomMemberEvent->senderId())->avatarUrl(this).isEmpty()) {
-            avatar_image = user(roomMemberEvent->senderId())->avatar(128, this);
-        } else {
-            qWarning() << "using this room's avatar";
-            avatar_image = avatar(128);
-        }
-        NotificationsManager::instance().postInviteNotification(this, displayName(), htmlSafeMemberName(roomMemberEvent->senderId()), avatar_image);
-    });
+            Q_EMIT canEncryptRoomChanged();
+            if (this->joinState() != JoinState::Invite) {
+                return;
+            }
+            auto roomMemberEvent = currentState().get<RoomMemberEvent>(localMember().id());
+
+            auto showNotification = [this, roomMemberEvent] {
+                QImage avatar_image;
+                if (roomMemberEvent && !member(roomMemberEvent->senderId()).avatarUrl().isEmpty()) {
+                    avatar_image = memberAvatar(roomMemberEvent->senderId()).get(this->connection(), 128, [] {});
+                } else {
+                    qWarning() << "using this room's avatar";
+                    avatar_image = avatar(128);
+                }
+
+                NotificationsManager::instance().postInviteNotification(this,
+                                                                        displayName(),
+                                                                        member(roomMemberEvent->senderId()).htmlSafeDisplayName(),
+                                                                        avatar_image);
+            };
+
+            if (NeoChatConfig::rejectUnknownInvites()) {
+                auto job = this->connection()->callApi<NeochatGetCommonRoomsJob>(roomMemberEvent->senderId());
+                connect(job, &BaseJob::result, this, [this, job, roomMemberEvent, showNotification] {
+                    QJsonObject replyData = job->jsonData();
+                    if (replyData.contains(QStringLiteral("joined"))) {
+                        const bool inAnyOfOurRooms = !replyData[QStringLiteral("joined")].toArray().isEmpty();
+                        if (inAnyOfOurRooms) {
+                            showNotification();
+                        } else {
+                            leaveRoom();
+                        }
+                    }
+                });
+            } else {
+                showNotification();
+            }
+        },
+        Qt::SingleShotConnection);
     connect(this, &Room::changed, this, [this] {
         Q_EMIT canEncryptRoomChanged();
         Q_EMIT parentIdsChanged();
@@ -255,28 +308,9 @@ void NeoChatRoom::forget()
     connection()->forgetRoom(id());
 }
 
-QVariantList NeoChatRoom::getUsersTyping() const
-{
-    auto users = usersTyping();
-    users.removeAll(localUser());
-    QVariantList userVariants;
-    for (const auto &user : users) {
-        if (connection()->isIgnored(user->id())) {
-            continue;
-        }
-        userVariants.append(QVariantMap{
-            {"id"_ls, user->id()},
-            {"avatarMediaId"_ls, user->avatarMediaId(this)},
-            {"displayName"_ls, user->displayname(this)},
-            {"display"_ls, user->name()},
-        });
-    }
-    return userVariants;
-}
-
 void NeoChatRoom::sendTypingNotification(bool isTyping)
 {
-    connection()->callApi<SetTypingJob>(BackgroundRequest, localUser()->id(), id(), isTyping, 10000);
+    connection()->callApi<SetTypingJob>(BackgroundRequest, localMember().id(), id(), isTyping, 10000);
 }
 
 const RoomEvent *NeoChatRoom::lastEvent() const
@@ -314,7 +348,7 @@ const RoomEvent *NeoChatRoom::lastEvent() const
             }
         }
 
-        if (connection()->isIgnored(user(event->senderId()))) {
+        if (connection()->isIgnored(event->senderId())) {
             continue;
         }
 
@@ -374,13 +408,13 @@ bool NeoChatRoom::isEventHighlighted(const RoomEvent *e) const
 
 void NeoChatRoom::checkForHighlights(const Quotient::TimelineItem &ti)
 {
-    auto localUserId = localUser()->id();
-    if (ti->senderId() == localUserId) {
+    auto localMember = this->localMember();
+    if (ti->senderId() == localMember.id()) {
         return;
     }
     if (auto *e = ti.viewAs<RoomMessageEvent>()) {
         const auto &text = e->plainBody();
-        if (text.contains(localUserId) || text.contains(safeMemberName(localUserId))) {
+        if (text.contains(localMember.id()) || text.contains(localMember.disambiguatedName())) {
             highlights.insert(e);
         }
     }
@@ -426,40 +460,6 @@ QDateTime NeoChatRoom::lastActiveTime()
     return messageEvents().rbegin()->get()->originTimestamp();
 }
 
-// An empty user is useful for returning as a model value to avoid properties being undefined.
-static const QVariantMap emptyUser = {
-    {"isLocalUser"_ls, false},
-    {"id"_ls, QString()},
-    {"displayName"_ls, QString()},
-    {"avatarSource"_ls, QUrl()},
-    {"avatarMediaId"_ls, QString()},
-    {"color"_ls, QColor()},
-    {"object"_ls, QVariant()},
-};
-
-QVariantMap NeoChatRoom::getUser(const QString &userID) const
-{
-    return getUser(user(userID));
-}
-
-QVariantMap NeoChatRoom::getUser(User *user) const
-{
-    if (user == nullptr) {
-        return emptyUser;
-    }
-
-    return QVariantMap{
-        {QStringLiteral("isLocalUser"), user->id() == localUser()->id()},
-        {QStringLiteral("id"), user->id()},
-        {QStringLiteral("displayName"), user->displayname(this)},
-        {QStringLiteral("escapedDisplayName"), htmlSafeMemberName(user->id())},
-        {QStringLiteral("avatarSource"), avatarForMember(user)},
-        {QStringLiteral("avatarMediaId"), user->avatarMediaId(this)},
-        {QStringLiteral("color"), Utils::getUserColor(user->hueF())},
-        {QStringLiteral("object"), QVariant::fromValue(user)},
-    };
-}
-
 QString NeoChatRoom::avatarMediaId() const
 {
     if (const auto avatar = Room::avatarMediaId(); !avatar.isEmpty()) {
@@ -467,10 +467,10 @@ QString NeoChatRoom::avatarMediaId() const
     }
 
     // Use the first (excluding self) user's avatar for direct chats
-    const auto dcUsers = directChatUsers();
-    for (const auto u : dcUsers) {
-        if (u != localUser()) {
-            return u->avatarMediaId(this);
+    const auto directChatMembers = this->directChatMembers();
+    for (const auto member : directChatMembers) {
+        if (member != localMember()) {
+            return member.avatarMediaId();
         }
     }
 
@@ -637,7 +637,7 @@ void NeoChatRoom::toggleReaction(const QString &eventId, const QString &reaction
                     continue;
                 }
 
-                if (e->senderId() == localUser()->id()) {
+                if (e->senderId() == localMember().id()) {
                     redactEventIds.push_back(e->id());
                     break;
                 }
@@ -666,7 +666,7 @@ bool NeoChatRoom::canSendEvent(const QString &eventType) const
         return false;
     }
     auto pl = plEvent->powerLevelForEvent(eventType);
-    auto currentPl = plEvent->powerLevelForUser(localUser()->id());
+    auto currentPl = plEvent->powerLevelForUser(localMember().id());
 
     return currentPl >= pl;
 }
@@ -678,7 +678,7 @@ bool NeoChatRoom::canSendState(const QString &eventType) const
         return false;
     }
     auto pl = plEvent->powerLevelForState(eventType);
-    auto currentPl = plEvent->powerLevelForUser(localUser()->id());
+    auto currentPl = plEvent->powerLevelForUser(localMember().id());
 
     return currentPl >= pl;
 }
@@ -856,7 +856,7 @@ void NeoChatRoom::setUrlPreviewEnabled(const bool &urlPreviewEnabled)
      *  "type": "org.matrix.room.preview_urls",
      * }
      */
-    connection()->callApi<SetAccountDataPerRoomJob>(localUser()->id(),
+    connection()->callApi<SetAccountDataPerRoomJob>(localMember().id(),
                                                     id(),
                                                     "org.matrix.room.preview_urls"_ls,
                                                     QJsonObject{{"disable"_ls, !urlPreviewEnabled}});
@@ -876,7 +876,7 @@ void NeoChatRoom::setUserPowerLevel(const QString &userID, const int &powerLevel
         qWarning() << "User is not a member of this room so power level cannot be set";
         return;
     }
-    int clampPowerLevel = std::clamp(powerLevel, 0, 100);
+    int clampPowerLevel = std::clamp(powerLevel, -1, 100);
 
     auto powerLevelContent = currentState().get("m.room.power_levels"_ls)->contentJson();
     auto powerLevelUserOverrides = powerLevelContent["users"_ls].toObject();
@@ -891,259 +891,18 @@ void NeoChatRoom::setUserPowerLevel(const QString &userID, const int &powerLevel
 
 int NeoChatRoom::getUserPowerLevel(const QString &userId) const
 {
-    auto powerLevelEvent = currentState().get<RoomPowerLevelsEvent>();
-    if (!powerLevelEvent) {
-        return 0;
-    }
-    return powerLevelEvent->powerLevelForUser(userId);
-}
-
-int NeoChatRoom::powerLevel(const QString &eventName, const bool &isStateEvent) const
-{
-    const auto powerLevelEvent = currentState().get<RoomPowerLevelsEvent>();
-    if (eventName == "ban"_ls) {
-        return powerLevelEvent->ban();
-    } else if (eventName == "kick"_ls) {
-        return powerLevelEvent->kick();
-    } else if (eventName == "invite"_ls) {
-        return powerLevelEvent->invite();
-    } else if (eventName == "redact"_ls) {
-        return powerLevelEvent->redact();
-    } else if (eventName == "users_default"_ls) {
-        return powerLevelEvent->usersDefault();
-    } else if (eventName == "state_default"_ls) {
-        return powerLevelEvent->stateDefault();
-    } else if (eventName == "events_default"_ls) {
-        return powerLevelEvent->eventsDefault();
-    } else if (isStateEvent) {
-        return powerLevelEvent->powerLevelForState(eventName);
-    } else {
-        return powerLevelEvent->powerLevelForEvent(eventName);
-    }
-}
-
-void NeoChatRoom::setPowerLevel(const QString &eventName, const int &newPowerLevel, const bool &isStateEvent)
-{
-    auto powerLevelContent = currentState().get("m.room.power_levels"_ls)->contentJson();
-    int clampPowerLevel = std::clamp(newPowerLevel, 0, 100);
-    int powerLevel = 0;
-
-    if (powerLevelContent.contains(eventName)) {
-        powerLevel = powerLevelContent[eventName].toInt();
-
-        if (powerLevel != clampPowerLevel) {
-            powerLevelContent[eventName] = clampPowerLevel;
-        }
-    } else {
-        auto eventPowerLevels = powerLevelContent["events"_ls].toObject();
-
-        if (eventPowerLevels.contains(eventName)) {
-            powerLevel = eventPowerLevels[eventName].toInt();
-        } else {
-            if (isStateEvent) {
-                powerLevel = powerLevelContent["state_default"_ls].toInt();
-            } else {
-                powerLevel = powerLevelContent["events_default"_ls].toInt();
-            }
-        }
-
-        if (powerLevel != clampPowerLevel) {
-            eventPowerLevels[eventName] = clampPowerLevel;
-            powerLevelContent["events"_ls] = eventPowerLevels;
-        }
+    if (!successorId().isEmpty()) {
+        return 0; // No one can upgrade a room that's already upgraded
     }
 
-    setState("m.room.power_levels"_ls, {}, powerLevelContent);
-}
-
-int NeoChatRoom::defaultUserPowerLevel() const
-{
-    return powerLevel("users_default"_ls);
-}
-
-void NeoChatRoom::setDefaultUserPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("users_default"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::invitePowerLevel() const
-{
-    return powerLevel("invite"_ls);
-}
-
-void NeoChatRoom::setInvitePowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("invite"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::kickPowerLevel() const
-{
-    return powerLevel("kick"_ls);
-}
-
-void NeoChatRoom::setKickPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("kick"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::banPowerLevel() const
-{
-    return powerLevel("ban"_ls);
-}
-
-void NeoChatRoom::setBanPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("ban"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::redactPowerLevel() const
-{
-    return powerLevel("redact"_ls);
-}
-
-void NeoChatRoom::setRedactPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("redact"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::statePowerLevel() const
-{
-    return powerLevel("state_default"_ls);
-}
-
-void NeoChatRoom::setStatePowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("state_default"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::defaultEventPowerLevel() const
-{
-    return powerLevel("events_default"_ls);
-}
-
-void NeoChatRoom::setDefaultEventPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("events_default"_ls, newPowerLevel);
-}
-
-int NeoChatRoom::powerLevelPowerLevel() const
-{
-    return powerLevel("m.room.power_levels"_ls, true);
-}
-
-void NeoChatRoom::setPowerLevelPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.power_levels"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::namePowerLevel() const
-{
-    return powerLevel("m.room.name"_ls, true);
-}
-
-void NeoChatRoom::setNamePowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.name"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::avatarPowerLevel() const
-{
-    return powerLevel("m.room.avatar"_ls, true);
-}
-
-void NeoChatRoom::setAvatarPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.avatar"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::canonicalAliasPowerLevel() const
-{
-    return powerLevel("m.room.canonical_alias"_ls, true);
-}
-
-void NeoChatRoom::setCanonicalAliasPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.canonical_alias"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::topicPowerLevel() const
-{
-    return powerLevel("m.room.topic"_ls, true);
-}
-
-void NeoChatRoom::setTopicPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.topic"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::encryptionPowerLevel() const
-{
-    return powerLevel("m.room.encryption"_ls, true);
-}
-
-void NeoChatRoom::setEncryptionPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.encryption"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::historyVisibilityPowerLevel() const
-{
-    return powerLevel("m.room.history_visibility"_ls, true);
-}
-
-void NeoChatRoom::setHistoryVisibilityPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.history_visibility"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::pinnedEventsPowerLevel() const
-{
-    return powerLevel("m.room.pinned_events"_ls, true);
-}
-
-void NeoChatRoom::setPinnedEventsPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.pinned_events"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::tombstonePowerLevel() const
-{
-    return powerLevel("m.room.tombstone"_ls, true);
-}
-
-void NeoChatRoom::setTombstonePowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.tombstone"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::serverAclPowerLevel() const
-{
-    return powerLevel("m.room.server_acl"_ls, true);
-}
-
-void NeoChatRoom::setServerAclPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.room.server_acl"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::spaceChildPowerLevel() const
-{
-    return powerLevel("m.space.child"_ls, true);
-}
-
-void NeoChatRoom::setSpaceChildPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.space.child"_ls, newPowerLevel, true);
-}
-
-int NeoChatRoom::spaceParentPowerLevel() const
-{
-    return powerLevel("m.space.parent"_ls, true);
-}
-
-void NeoChatRoom::setSpaceParentPowerLevel(const int &newPowerLevel)
-{
-    setPowerLevel("m.space.parent"_ls, newPowerLevel, true);
+    const auto &mId = userId.isEmpty() ? connection()->userId() : userId;
+    if (const auto *plEvent = currentState().get<RoomPowerLevelsEvent>()) {
+        return plEvent->powerLevelForUser(mId);
+    }
+    if (const auto *createEvent = creation()) {
+        return createEvent->senderId() == mId ? 100 : 0;
+    }
+    return 0; // That's rather weird but may happen, according to rvdh
 }
 
 QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user, QString reason)
@@ -1156,7 +915,11 @@ QCoro::Task<void> NeoChatRoom::doDeleteMessagesByUser(const QString &user, QStri
     }
     for (const auto &e : events) {
         auto job = connection()->callApi<RedactEventJob>(id(), QString::fromLatin1(QUrl::toPercentEncoding(e)), connection()->generateTxnId(), reason);
+#if Quotient_VERSION_MINOR > 8
+        co_await qCoro(job.get(), &BaseJob::finished);
+#else
         co_await qCoro(job, &BaseJob::finished);
+#endif
         if (job->error() != BaseJob::Success) {
             qWarning() << "Error: \"" << job->error() << "\" while deleting messages. Aborting";
             break;
@@ -1575,7 +1338,6 @@ void NeoChatRoom::setPushNotificationState(PushNotificationState::State state)
 
     m_currentPushNotificationState = state;
     Q_EMIT pushNotificationStateChanged(m_currentPushNotificationState);
-
 }
 
 void NeoChatRoom::updatePushNotificationState(QString type)
@@ -1636,10 +1398,9 @@ void NeoChatRoom::updatePushNotificationState(QString type)
 void NeoChatRoom::reportEvent(const QString &eventId, const QString &reason)
 {
     auto job = connection()->callApi<ReportContentJob>(id(), eventId, -50, reason);
-    connect(job, &BaseJob::finished, this, [this, job]() {
+    connect(job, &BaseJob::finished, this, [job]() {
         if (job->error() == BaseJob::Success) {
-            Q_EMIT showMessage(Positive, i18n("Report sent successfully."));
-            Q_EMIT showMessage(MessageType::Positive, i18n("Report sent successfully."));
+            Q_EMIT Controller::instance().showMessage(Controller::Positive, i18n("Report sent successfully."));
         }
     });
 }
@@ -1660,7 +1421,7 @@ void NeoChatRoom::openEventMediaExternally(const QString &eventId)
     if (evtIt != messageEvents().rend() && is<RoomMessageEvent>(**evtIt)) {
         const auto event = evtIt->viewAs<RoomMessageEvent>();
         if (event->hasFileContent()) {
-            const auto transferInfo = fileTransferInfo(eventId);
+            const auto transferInfo = cachedFileTransferInfo(event);
             if (transferInfo.completed()) {
                 UrlHelper helper;
                 helper.openUrl(transferInfo.localPath);
@@ -1668,15 +1429,20 @@ void NeoChatRoom::openEventMediaExternally(const QString &eventId)
                 downloadFile(eventId,
                              QUrl(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + u'/'
                                   + event->id().replace(u':', u'_').replace(u'/', u'_').replace(u'+', u'_') + fileNameToDownload(eventId)));
-                connect(this, &Room::fileTransferCompleted, this, [this, eventId](QString id, QUrl localFile, FileSourceInfo fileMetadata) {
-                    Q_UNUSED(localFile);
-                    Q_UNUSED(fileMetadata);
-                    if (id == eventId) {
-                        auto transferInfo = fileTransferInfo(eventId);
-                        UrlHelper helper;
-                        helper.openUrl(transferInfo.localPath);
-                    }
-                });
+                connect(
+                    this,
+                    &Room::fileTransferCompleted,
+                    this,
+                    [this, eventId](QString id, QUrl localFile, FileSourceInfo fileMetadata) {
+                        Q_UNUSED(localFile);
+                        Q_UNUSED(fileMetadata);
+                        if (id == eventId) {
+                            auto transferInfo = fileTransferInfo(eventId);
+                            UrlHelper helper;
+                            helper.openUrl(transferInfo.localPath);
+                        }
+                    },
+                    static_cast<Qt::ConnectionType>(Qt::SingleShotConnection));
             }
         }
     }
@@ -1696,20 +1462,66 @@ void NeoChatRoom::copyEventMedia(const QString &eventId)
                 downloadFile(eventId,
                              QUrl(QStandardPaths::writableLocation(QStandardPaths::CacheLocation) + u'/'
                                   + event->id().replace(u':', u'_').replace(u'/', u'_').replace(u'+', u'_') + fileNameToDownload(eventId)));
-                connect(this, &Room::fileTransferCompleted, this, [this, eventId](QString id, QUrl localFile, FileSourceInfo fileMetadata) {
-                    Q_UNUSED(localFile);
-                    Q_UNUSED(fileMetadata);
-                    if (id == eventId) {
-                        auto transferInfo = fileTransferInfo(eventId);
-                        Clipboard clipboard;
-                        clipboard.setImage(transferInfo.localPath);
-                    }
-                });
+                connect(
+                    this,
+                    &Room::fileTransferCompleted,
+                    this,
+                    [this, eventId](QString id, QUrl localFile, FileSourceInfo fileMetadata) {
+                        Q_UNUSED(localFile);
+                        Q_UNUSED(fileMetadata);
+                        if (id == eventId) {
+                            auto transferInfo = fileTransferInfo(eventId);
+                            Clipboard clipboard;
+                            clipboard.setImage(transferInfo.localPath);
+                        }
+                    },
+                    static_cast<Qt::ConnectionType>(Qt::SingleShotConnection));
             }
         }
     }
 }
 
+FileTransferInfo NeoChatRoom::cachedFileTransferInfo(const Quotient::RoomEvent *event) const
+{
+    QString mxcUrl;
+    int total = 0;
+    if (auto evt = eventCast<const Quotient::RoomMessageEvent>(event)) {
+        if (evt->hasFileContent()) {
+            mxcUrl = evt->content()->fileInfo()->url().toString();
+            total = evt->content()->fileInfo()->payloadSize;
+        }
+    } else if (auto evt = eventCast<const Quotient::StickerEvent>(event)) {
+        mxcUrl = evt->image().fileInfo()->url().toString();
+        total = evt->image().fileInfo()->payloadSize;
+    }
+
+    FileTransferInfo transferInfo = fileTransferInfo(event->id());
+    if (transferInfo.active()) {
+        return transferInfo;
+    }
+
+    auto config = KSharedConfig::openStateConfig(QStringLiteral("neochatdownloads"))->group(QStringLiteral("downloads"));
+    if (!config.hasKey(mxcUrl.mid(6))) {
+        return transferInfo;
+    }
+
+    const auto path = config.readPathEntry(mxcUrl.mid(6), QString());
+    QFileInfo info(path);
+    if (!info.isFile()) {
+        config.deleteEntry(mxcUrl);
+        return transferInfo;
+    }
+    // TODO: we could check the hash here
+    return FileTransferInfo{
+        .status = FileTransferInfo::Completed,
+        .isUpload = false,
+        .progress = total,
+        .total = total,
+        .localDir = QUrl(info.dir().path()),
+        .localPath = QUrl::fromLocalFile(path),
+    };
+}
+
 ChatBarCache *NeoChatRoom::mainCache() const
 {
     return m_mainCache;
@@ -1768,7 +1580,7 @@ void NeoChatRoom::editLastMessage()
         }
 
         // check if the current message's sender's id is same as the user's id
-        if ((*it)->senderId() == localUser()->id()) {
+        if ((*it)->senderId() == localMember().id()) {
             auto content = (*it)->contentJson();
 
             if (e->msgtype() != MessageEventType::Unknown) {
@@ -1893,13 +1705,9 @@ int NeoChatRoom::maxRoomVersion() const
     return maxVersion;
 }
 
-Quotient::User *NeoChatRoom::directChatRemoteUser() const
+Quotient::RoomMember NeoChatRoom::directChatRemoteMember() const
 {
-    auto users = connection()->directChatUsers(this);
-    if (users.isEmpty()) {
-        return nullptr;
-    }
-    return users[0];
+    return directChatMembers()[0];
 }
 
 void NeoChatRoom::sendLocation(float lat, float lon, const QString &description)
@@ -1931,18 +1739,38 @@ QByteArray NeoChatRoom::roomAcountDataJson(const QString &eventType)
     return QJsonDocument(accountData(eventType)->fullJson()).toJson();
 }
 
-QUrl NeoChatRoom::avatarForMember(Quotient::User *user) const
+void NeoChatRoom::downloadEventFromServer(const QString &eventId)
 {
-    const auto &url = memberAvatarUrl(user->id());
-    if (url.isEmpty() || url.scheme() != "mxc"_ls) {
-        return {};
+    if (findInTimeline(eventId) != historyEdge()) {
+        return;
     }
-    auto avatar = connection()->makeMediaUrl(url);
-    if (avatar.isValid() && avatar.scheme() == QStringLiteral("mxc")) {
-        return avatar;
-    } else {
-        return QUrl();
+    auto job = connection()->callApi<GetOneRoomEventJob>(id(), eventId);
+    connect(job, &BaseJob::success, this, [this, job, eventId] {
+        // The event may have arrived in the meantime so check it's not in the timeline.
+        if (findInTimeline(eventId) != historyEdge()) {
+            return;
+        }
+
+        event_ptr_tt<RoomEvent> event = fromJson<event_ptr_tt<RoomEvent>>(job->jsonData());
+        m_extraEvents.push_back(std::move(event));
+        Q_EMIT extraEventLoaded(eventId);
+    });
+}
+
+const RoomEvent *NeoChatRoom::getEvent(const QString &eventId) const
+{
+    if (eventId.isEmpty()) {
+        return nullptr;
     }
+    const auto timelineIt = findInTimeline(eventId);
+    if (timelineIt != historyEdge()) {
+        return timelineIt->get();
+    }
+
+    auto extraIt = std::find_if(m_extraEvents.begin(), m_extraEvents.end(), [eventId](const Quotient::event_ptr_tt<Quotient::RoomEvent> &event) {
+        return event->id() == eventId;
+    });
+    return extraIt != m_extraEvents.end() ? extraIt->get() : nullptr;
 }
 
 const RoomEvent *NeoChatRoom::getReplyForEvent(const RoomEvent &event) const
@@ -1965,18 +1793,26 @@ const RoomEvent *NeoChatRoom::getReplyForEvent(const RoomEvent &event) const
     return replyPtr;
 }
 
-void NeoChatRoom::loadReply(const QString &eventId, const QString &replyId)
+void NeoChatRoom::cleanupExtraEventRange(Quotient::RoomEventsRange events)
 {
-    auto job = connection()->callApi<GetOneRoomEventJob>(id(), replyId);
-    connect(job, &BaseJob::success, this, [this, job, eventId, replyId] {
-        m_extraEvents.push_back(fromJson<event_ptr_tt<RoomEvent>>(job->jsonData()));
-        Q_EMIT replyLoaded(eventId, replyId);
-    });
+    for (auto &&event : events) {
+        cleanupExtraEvent(event->id());
+    }
 }
 
-User *NeoChatRoom::invitingUser() const
+void NeoChatRoom::cleanupExtraEvent(const QString &eventId)
 {
-    return connection()->user(currentState().get<RoomMemberEvent>(connection()->userId())->senderId());
+    auto it = std::find_if(m_extraEvents.begin(), m_extraEvents.end(), [eventId](Quotient::event_ptr_tt<Quotient::RoomEvent> &event) {
+        return event->id() == eventId;
+    });
+
+    if (it != m_extraEvents.end()) {
+        m_extraEvents.erase(it);
+    }
+}
+QString NeoChatRoom::invitingUserId() const
+{
+    return currentState().get<RoomMemberEvent>(connection()->userId())->senderId();
 }
 
 void NeoChatRoom::setRoomState(const QString &type, const QString &stateKey, const QByteArray &content)
@@ -1984,4 +1820,13 @@ void NeoChatRoom::setRoomState(const QString &type, const QString &stateKey, con
     setState(type, stateKey, QJsonDocument::fromJson(content).object());
 }
 
+#if Quotient_VERSION_MINOR == 8
+QList<RoomMember> NeoChatRoom::otherMembersTyping() const
+{
+    auto memberTyping = membersTyping();
+    memberTyping.removeAll(localMember());
+    return memberTyping;
+}
+#endif
+
 #include "moc_neochatroom.cpp"
