@@ -8,13 +8,11 @@
 #include <QCache>
 #include <QObject>
 #include <QQmlEngine>
-#include <QTextCursor>
 
 #include <QCoroTask>
-#include <Quotient/user.h>
+#include <Quotient/roommember.h>
 
 #include "enums/pushrule.h"
-#include "events/pollevent.h"
 #include "pollhandler.h"
 
 namespace Quotient
@@ -42,27 +40,6 @@ class NeoChatRoom : public Quotient::Room
     QML_ELEMENT
     QML_UNCREATABLE("")
 
-    /**
-     * @brief A list of users currently typing in the room.
-     *
-     * The list does not include the local user.
-     *
-     * This is different to getting a list of Quotient::User objects
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room. This function
-     * provides the room context and puts the result as a list of QVariantMap objects.
-     *
-     * @return a QVariantMap for the user with the following
-     * parameters:
-     *  - id - User ID.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - displayName - Display name in the context of this room.
-     *  - display - Name in the context of this room.
-     *
-     * @sa Quotient::User
-     */
-    Q_PROPERTY(QVariantList usersTyping READ getUsersTyping NOTIFY typingChanged)
-
     /**
      * @brief Convenience function to get the QDateTime of the last event.
      *
@@ -95,9 +72,9 @@ class NeoChatRoom : public Quotient::Room
     Q_PROPERTY(QString avatarMediaId READ avatarMediaId NOTIFY avatarChanged STORED false)
 
     /**
-     * @brief Get a user object for the other person in a direct chat.
+     * @brief Get a RoomMember object for the other person in a direct chat.
      */
-    Q_PROPERTY(Quotient::User *directChatRemoteUser READ directChatRemoteUser CONSTANT)
+    Q_PROPERTY(Quotient::RoomMember directChatRemoteMember READ directChatRemoteMember CONSTANT)
 
     /**
      * @brief The Matrix IDs of this room's parents.
@@ -214,101 +191,6 @@ class NeoChatRoom : public Quotient::Room
      */
     Q_PROPERTY(bool canEncryptRoom READ canEncryptRoom NOTIFY canEncryptRoomChanged)
 
-    /**
-     * @brief The default power level in the room for new users.
-     */
-    Q_PROPERTY(int defaultUserPowerLevel READ defaultUserPowerLevel WRITE setDefaultUserPowerLevel NOTIFY defaultUserPowerLevelChanged)
-
-    /**
-     * @brief The power level required to invite users to the room.
-     */
-    Q_PROPERTY(int invitePowerLevel READ invitePowerLevel WRITE setInvitePowerLevel NOTIFY invitePowerLevelChanged)
-
-    /**
-     * @brief The power level required to kick users from the room.
-     */
-    Q_PROPERTY(int kickPowerLevel READ kickPowerLevel WRITE setKickPowerLevel NOTIFY kickPowerLevelChanged)
-
-    /**
-     * @brief The power level required to ban users from the room.
-     */
-    Q_PROPERTY(int banPowerLevel READ banPowerLevel WRITE setBanPowerLevel NOTIFY banPowerLevelChanged)
-
-    /**
-     * @brief The power level required to delete other user messages.
-     */
-    Q_PROPERTY(int redactPowerLevel READ redactPowerLevel WRITE setRedactPowerLevel NOTIFY redactPowerLevelChanged)
-
-    /**
-     * @brief The default power level for state events that are not explicitly specified.
-     */
-    Q_PROPERTY(int statePowerLevel READ statePowerLevel WRITE setStatePowerLevel NOTIFY statePowerLevelChanged)
-
-    /**
-     * @brief The default power level for event that are not explicitly specified.
-     */
-    Q_PROPERTY(int defaultEventPowerLevel READ defaultEventPowerLevel WRITE setDefaultEventPowerLevel NOTIFY defaultEventPowerLevelChanged)
-
-    /**
-     * @brief The power level required to change power levels for the room.
-     */
-    Q_PROPERTY(int powerLevelPowerLevel READ powerLevelPowerLevel WRITE setPowerLevelPowerLevel NOTIFY powerLevelPowerLevelChanged)
-
-    /**
-     * @brief The power level required to change the room name.
-     */
-    Q_PROPERTY(int namePowerLevel READ namePowerLevel WRITE setNamePowerLevel NOTIFY namePowerLevelChanged)
-
-    /**
-     * @brief The power level required to change the room avatar.
-     */
-    Q_PROPERTY(int avatarPowerLevel READ avatarPowerLevel WRITE setAvatarPowerLevel NOTIFY avatarPowerLevelChanged)
-
-    /**
-     * @brief The power level required to change the room aliases.
-     */
-    Q_PROPERTY(int canonicalAliasPowerLevel READ canonicalAliasPowerLevel WRITE setCanonicalAliasPowerLevel NOTIFY canonicalAliasPowerLevelChanged)
-
-    /**
-     * @brief The power level required to change the room topic.
-     */
-    Q_PROPERTY(int topicPowerLevel READ topicPowerLevel WRITE setTopicPowerLevel NOTIFY topicPowerLevelChanged)
-
-    /**
-     * @brief The power level required to encrypt the room.
-     */
-    Q_PROPERTY(int encryptionPowerLevel READ encryptionPowerLevel WRITE setEncryptionPowerLevel NOTIFY encryptionPowerLevelChanged)
-
-    /**
-     * @brief The power level required to change the room history visibility.
-     */
-    Q_PROPERTY(int historyVisibilityPowerLevel READ historyVisibilityPowerLevel WRITE setHistoryVisibilityPowerLevel NOTIFY historyVisibilityPowerLevelChanged)
-
-    /**
-     * @brief The power level required to pin events in the room.
-     */
-    Q_PROPERTY(int pinnedEventsPowerLevel READ pinnedEventsPowerLevel WRITE setPinnedEventsPowerLevel NOTIFY pinnedEventsPowerLevelChanged)
-
-    /**
-     * @brief The power level required to upgrade the room.
-     */
-    Q_PROPERTY(int tombstonePowerLevel READ tombstonePowerLevel WRITE setTombstonePowerLevel NOTIFY tombstonePowerLevelChanged)
-
-    /**
-     * @brief The power level required to set the room server access control list (ACL).
-     */
-    Q_PROPERTY(int serverAclPowerLevel READ serverAclPowerLevel WRITE setServerAclPowerLevel NOTIFY serverAclPowerLevelChanged)
-
-    /**
-     * @brief The power level required to add children to a space.
-     */
-    Q_PROPERTY(int spaceChildPowerLevel READ spaceChildPowerLevel WRITE setSpaceChildPowerLevel NOTIFY spaceChildPowerLevelChanged)
-
-    /**
-     * @brief The power level required to set the room parent space.
-     */
-    Q_PROPERTY(int spaceParentPowerLevel READ spaceParentPowerLevel WRITE setSpaceParentPowerLevel NOTIFY spaceParentPowerLevelChanged)
-
     /**
      * @brief The cache for the main chat bar in the room.
      */
@@ -319,73 +201,13 @@ class NeoChatRoom : public Quotient::Room
      */
     Q_PROPERTY(ChatBarCache *editCache READ editCache CONSTANT)
 
+#if Quotient_VERSION_MINOR == 8
+    Q_PROPERTY(QList<Quotient::RoomMember> otherMembersTyping READ otherMembersTyping NOTIFY typingChanged)
+#endif
+
 public:
-    /**
-     * @brief Define the types on inline messages that can be shown.
-     */
-    enum MessageType {
-        Positive, /**< Positive message, typically green. */
-        Info, /**< Info message, typically highlight color. */
-        Error, /**< Error message, typically red. */
-    };
-    Q_ENUM(MessageType)
-
     explicit NeoChatRoom(Quotient::Connection *connection, QString roomId, Quotient::JoinState joinState = {});
 
-    /**
-     * @brief Get a user in the context of this room.
-     *
-     * This is different to getting a Quotient::User object
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room. This function
-     * provides the room context and outputs the result as QVariantMap.
-     *
-     * Can be called with an empty QString to return an empty user, which is a useful return
-     * from models to avoid undefined properties.
-     *
-     * @param userID the ID of the user to output.
-     *
-     * @return a QVariantMap for the user with the following properties:
-     *  - isLocalUser - Whether the user is the local user.
-     *  - id - The matrix ID of the user.
-     *  - displayName - Display name in the context of this room.
-     *  - avatarSource - The mxc URL for the user's avatar in the current room.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - color - Color for the user.
-     *  - object - The Quotient::User object for the user.
-     *
-     * @sa Quotient::User
-     */
-    Q_INVOKABLE [[nodiscard]] QVariantMap getUser(const QString &userID) const;
-
-    /**
-     * @brief Get a user in the context of this room.
-     *
-     * This is different to getting a Quotient::User object
-     * as neither of those can provide details like the displayName or avatarMediaId
-     * without the room context as these can vary from room to room. This function
-     * provides the room context and outputs the result as QVariantMap.
-     *
-     * Can be called with a nullptr to return an empty user, which is a useful return
-     * from models to avoid undefined properties.
-     *
-     * @param user the user to output.
-     *
-     * @return a QVariantMap for the user with the following properties:
-     *  - isLocalUser - Whether the user is the local user.
-     *  - id - The matrix ID of the user.
-     *  - displayName - Display name in the context of this room.
-     *  - avatarSource - The mxc URL for the user's avatar in the current room.
-     *  - avatarMediaId - Avatar id in the context of this room.
-     *  - color - Color for the user.
-     *  - object - The Quotient::User object for the user.
-     *
-     * @sa Quotient::User
-     */
-    Q_INVOKABLE [[nodiscard]] QVariantMap getUser(Quotient::User *user) const;
-
-    [[nodiscard]] QVariantList getUsersTyping() const;
-
     [[nodiscard]] QDateTime lastActiveTime();
 
     /**
@@ -497,7 +319,7 @@ public:
 
     [[nodiscard]] QString avatarMediaId() const;
 
-    Quotient::User *directChatRemoteUser() const;
+    Quotient::RoomMember directChatRemoteMember() const;
 
     /**
      * @brief Whether this room has one or more parent spaces set.
@@ -685,66 +507,6 @@ public:
 
     Q_INVOKABLE void setUserPowerLevel(const QString &userID, const int &powerLevel);
 
-    [[nodiscard]] int powerLevel(const QString &eventName, const bool &isStateEvent = false) const;
-    void setPowerLevel(const QString &eventName, const int &newPowerLevel, const bool &isStateEvent = false);
-
-    [[nodiscard]] int defaultUserPowerLevel() const;
-    void setDefaultUserPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int invitePowerLevel() const;
-    void setInvitePowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int kickPowerLevel() const;
-    void setKickPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int banPowerLevel() const;
-    void setBanPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int redactPowerLevel() const;
-    void setRedactPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int statePowerLevel() const;
-    void setStatePowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int defaultEventPowerLevel() const;
-    void setDefaultEventPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int powerLevelPowerLevel() const;
-    void setPowerLevelPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int namePowerLevel() const;
-    void setNamePowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int avatarPowerLevel() const;
-    void setAvatarPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int canonicalAliasPowerLevel() const;
-    void setCanonicalAliasPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int topicPowerLevel() const;
-    void setTopicPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int encryptionPowerLevel() const;
-    void setEncryptionPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int historyVisibilityPowerLevel() const;
-    void setHistoryVisibilityPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int pinnedEventsPowerLevel() const;
-    void setPinnedEventsPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int tombstonePowerLevel() const;
-    void setTombstonePowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int serverAclPowerLevel() const;
-    void setServerAclPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int spaceChildPowerLevel() const;
-    void setSpaceChildPowerLevel(const int &newPowerLevel);
-
-    [[nodiscard]] int spaceParentPowerLevel() const;
-    void setSpaceParentPowerLevel(const int &newPowerLevel);
-
     ChatBarCache *mainCache() const;
 
     ChatBarCache *editCache() const;
@@ -787,24 +549,43 @@ public:
      */
     Q_INVOKABLE QByteArray roomAcountDataJson(const QString &eventType);
 
-    Q_INVOKABLE [[nodiscard]] QUrl avatarForMember(Quotient::User *user) const;
+    /**
+     * @brief Loads the event with the given id from the server and saves it locally.
+     *
+     * Intended to retrieve events that are needed, e.g. replied to events that are
+     * not currently in the timeline.
+     *
+     * If the event is already in the timeline nothing will happen.
+     */
+    void downloadEventFromServer(const QString &eventId);
+
+    /**
+     * @brief Returns the event with the given ID if available.
+     *
+     * This function will check both the timeline and extra events and return a
+     * non-nullptr value if it is found in either.
+     *
+     * The result will be nullptr if not found so needs to be managed.
+     */
+    const Quotient::RoomEvent *getEvent(const QString &eventId) const;
 
     /**
      * @brief Returns the event that is being replied to. This includes events that were manually loaded using NeoChatRoom::loadReply.
      */
     const Quotient::RoomEvent *getReplyForEvent(const Quotient::RoomEvent &event) const;
 
-    /**
-     * Loads the event replyId with the given id from the server and saves it locally.
-     * For models to update correctly, eventId must be the event that is replying to replyId.
-     * Intended to load the replied-to event when it isn't available locally.
-     */
-    Q_INVOKABLE void loadReply(const QString &eventId, const QString &replyId);
-
     /**
      * If we're invited to this room, the user that invited us. Undefined in other cases.
      */
-    Q_INVOKABLE Quotient::User *invitingUser() const;
+    Q_INVOKABLE QString invitingUserId() const;
+
+    /**
+     * @brief Return the cached file transfer information for the event.
+     *
+     * If we downloaded the file previously, return a struct with Completed status
+     * and the local file path stored in KSharedCOnfig
+     */
+    Quotient::FileTransferInfo cachedFileTransferInfo(const Quotient::RoomEvent *event) const;
 
 private:
     QSet<const Quotient::RoomEvent *> highlights;
@@ -831,6 +612,12 @@ private:
 
     QCache<QString, PollHandler> m_polls;
     std::vector<Quotient::event_ptr_tt<Quotient::RoomEvent>> m_extraEvents;
+    void cleanupExtraEventRange(Quotient::RoomEventsRange events);
+    void cleanupExtraEvent(const QString &eventId);
+
+#if Quotient_VERSION_MINOR == 8
+    QList<Quotient::RoomMember> otherMembersTyping() const;
+#endif
 
 private Q_SLOTS:
     void updatePushNotificationState(QString type);
@@ -853,33 +640,13 @@ Q_SIGNALS:
     void readOnlyChanged();
     void displayNameChanged();
     void pushNotificationStateChanged(PushNotificationState::State state);
-    void showMessage(MessageType messageType, const QString &message);
     void canEncryptRoomChanged();
     void joinRuleChanged();
     void historyVisibilityChanged();
     void defaultUrlPreviewStateChanged();
     void urlPreviewEnabledChanged();
     void maxRoomVersionChanged();
-    void defaultUserPowerLevelChanged();
-    void invitePowerLevelChanged();
-    void kickPowerLevelChanged();
-    void banPowerLevelChanged();
-    void redactPowerLevelChanged();
-    void statePowerLevelChanged();
-    void defaultEventPowerLevelChanged();
-    void powerLevelPowerLevelChanged();
-    void namePowerLevelChanged();
-    void avatarPowerLevelChanged();
-    void canonicalAliasPowerLevelChanged();
-    void topicPowerLevelChanged();
-    void encryptionPowerLevelChanged();
-    void historyVisibilityPowerLevelChanged();
-    void pinnedEventsPowerLevelChanged();
-    void tombstonePowerLevelChanged();
-    void serverAclPowerLevelChanged();
-    void spaceChildPowerLevelChanged();
-    void spaceParentPowerLevelChanged();
-    void replyLoaded(const QString &eventId, const QString &replyId);
+    void extraEventLoaded(const QString &eventId);
 
 public Q_SLOTS:
     /**
