@@ -39,6 +39,7 @@ RoomManager::RoomManager(QObject *parent)
     , m_timelineModel(new TimelineModel(this))
     , m_messageFilterModel(new MessageFilterModel(this, m_timelineModel))
     , m_mediaMessageFilterModel(new MediaMessageFilterModel(this, m_messageFilterModel))
+    , m_userListModel(new UserListModel(this))
 {
     m_lastRoomConfig = m_config->group(QStringLiteral("LastOpenRoom"));
     m_lastSpaceConfig = m_config->group(QStringLiteral("LastOpenSpace"));
@@ -46,6 +47,7 @@ RoomManager::RoomManager(QObject *parent)
 
     connect(this, &RoomManager::currentRoomChanged, this, [this]() {
         m_timelineModel->setRoom(m_currentRoom);
+        m_userListModel->setRoom(m_currentRoom);
     });
 
     connect(&Controller::instance(), &Controller::activeConnectionChanged, this, [this](NeoChatConnection *connection) {
@@ -113,6 +115,16 @@ MediaMessageFilterModel *RoomManager::mediaMessageFilterModel() const
     return m_mediaMessageFilterModel;
 }
 
+UserListModel *RoomManager::userListModel() const
+{
+    return m_userListModel;
+}
+
+void RoomManager::activateUserModel()
+{
+    m_userListModel->activate();
+}
+
 UriResolveResult RoomManager::resolveResource(const Uri &uri)
 {
     return UriResolverBase::visitResource(m_connection, uri);
@@ -146,13 +158,20 @@ void RoomManager::resolveResource(const QString &idOrUri, const QString &action)
         if ((uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) && action != "no_join"_ls) {
             Q_EMIT askJoinRoom(uri.primaryId());
         }
-    } else { // Invalid cases should have been eliminated earlier
-        Q_ASSERT(result == Quotient::UriResolved);
+    } else {
+        if (result != Quotient::UriResolved) {
+            return;
+        }
 
         if (uri.type() == Uri::RoomAlias || uri.type() == Uri::RoomId) {
-            connectSingleShot(m_connection.get(), &NeoChatConnection::newRoom, this, [this, uri](Room *room) {
-                resolveResource(room->id());
-            });
+            connect(
+                m_connection.get(),
+                &NeoChatConnection::newRoom,
+                this,
+                [this, uri](Room *room) {
+                    resolveResource(room->id());
+                },
+                Qt::SingleShotConnection);
         }
     }
 }
@@ -165,7 +184,7 @@ void RoomManager::maximizeMedia(int index)
     Q_EMIT showMaximizedMedia(index);
 }
 
-void RoomManager::maximizeCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language)
+void RoomManager::maximizeCode(const RoomMember &author, const QDateTime &time, const QString &codeText, const QString &language)
 {
     if (codeText.isEmpty()) {
         return;
@@ -237,22 +256,25 @@ void RoomManager::loadInitialRoom()
 void RoomManager::openRoomForActiveConnection()
 {
     if (!m_connection) {
-        m_currentRoom = nullptr;
+        setCurrentRoom({});
+        setCurrentSpace({}, false);
+        return;
     }
-    if (m_lastRoomConfig.readEntry(m_connection->userId(), QString()).isEmpty()) {
+    const auto &lastRoom = m_lastRoomConfig.readEntry(m_connection->userId(), QString());
+    if (lastRoom.isEmpty() || !m_connection->room(lastRoom)) {
         setCurrentRoom({});
     } else {
-        resolveResource(m_lastRoomConfig.readEntry(m_connection->userId(), QString()));
+        m_currentRoom = nullptr;
+        resolveResource(lastRoom);
     }
     setCurrentSpace(m_lastSpaceConfig.readEntry(m_connection->userId(), QString()), false);
 }
 
 UriResolveResult RoomManager::visitUser(User *user, const QString &action)
 {
-    if (action == "mention"_ls || action.isEmpty()) {
-        // send it has QVariantMap because the properties in the
+    if (action == "mention"_ls || action == "qr"_ls || action.isEmpty()) {
         user->load();
-        Q_EMIT showUserDetail(user);
+        Q_EMIT showUserDetail(user, action == "qr"_ls ? nullptr : currentRoom());
     } else if (action == "_interactive"_ls) {
         user->requestDirectChat();
     } else if (action == "chat"_ls) {
@@ -301,33 +323,69 @@ void RoomManager::visitRoom(Room *r, const QString &eventId)
 void RoomManager::joinRoom(Quotient::Connection *account, const QString &roomAliasOrId, const QStringList &viaServers)
 {
     auto job = account->joinRoom(roomAliasOrId, viaServers);
-    connectSingleShot(job, &Quotient::BaseJob::finished, this, [this, account](Quotient::BaseJob *finish) {
-        if (finish->status() == Quotient::BaseJob::Success) {
-            connectSingleShot(account, &NeoChatConnection::newRoom, this, [this](Quotient::Room *room) {
-                resolveResource(room->id());
-            });
-        } else {
-            Q_EMIT warning(i18n("Failed to join room"), finish->errorString());
-        }
-    });
+#if Quotient_VERSION_MINOR > 8
+    connect(
+        job.get(),
+        &Quotient::BaseJob::finished,
+        this,
+        [this, account](Quotient::BaseJob *finish) {
+#else
+    connect(
+        job,
+        &Quotient::BaseJob::finished,
+        this,
+        [this, account](Quotient::BaseJob *finish) {
+#endif
+            if (finish->status() == Quotient::BaseJob::Success) {
+                connect(
+                    account,
+                    &NeoChatConnection::newRoom,
+                    this,
+                    [this](Quotient::Room *room) {
+                        resolveResource(room->id());
+                    },
+                    Qt::SingleShotConnection);
+            } else {
+                Q_EMIT warning(i18n("Failed to join room"), finish->errorString());
+            }
+        },
+        Qt::SingleShotConnection);
 }
 
 void RoomManager::knockRoom(NeoChatConnection *account, const QString &roomAliasOrId, const QString &reason, const QStringList &viaServers)
 {
-    auto *const job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, reason);
+    auto job = account->callApi<KnockRoomJob>(roomAliasOrId, viaServers, reason);
     // Upon completion, ensure a room object is created in case it hasn't come
     // with a sync yet. If the room object is not there, provideRoom() will
     // create it in Join state. finished() is used here instead of success()
     // to overtake clients that may add their own slots to finished().
-    connectSingleShot(job, &BaseJob::finished, this, [this, job, account] {
-        if (job->status() == Quotient::BaseJob::Success) {
-            connectSingleShot(account, &NeoChatConnection::newRoom, this, [this](Quotient::Room *room) {
-                Q_EMIT currentRoom()->showMessage(NeoChatRoom::Info, i18n("You requested to join '%1'", room->name()));
-            });
-        } else {
-            Q_EMIT warning(i18n("Failed to request joining room"), job->errorString());
-        }
-    });
+#if Quotient_VERSION_MINOR > 8
+    connect(
+        job.get(),
+        &BaseJob::finished,
+        this,
+        [this, job, account] {
+#else
+    connect(
+        job,
+        &BaseJob::finished,
+        this,
+        [this, job, account] {
+#endif
+            if (job->status() == Quotient::BaseJob::Success) {
+                connect(
+                    account,
+                    &NeoChatConnection::newRoom,
+                    this,
+                    [this](Quotient::Room *room) {
+                        Q_EMIT Controller::instance().showMessage(Controller::Info, i18n("You requested to join '%1'", room -> name()));
+                    },
+                    Qt::SingleShotConnection);
+            } else {
+                Q_EMIT warning(i18n("Failed to request joining room"), job->errorString());
+            }
+        },
+        Qt::SingleShotConnection);
 }
 
 bool RoomManager::visitNonMatrix(const QUrl &url)
@@ -383,7 +441,9 @@ void RoomManager::setCurrentSpace(const QString &spaceId, bool setRoom)
     m_sortFilterRoomTreeModel->setMode(m_currentSpaceId == QLatin1String("DM") ? SortFilterRoomTreeModel::DirectChats : SortFilterRoomTreeModel::Rooms);
 
     Q_EMIT currentSpaceChanged();
-    m_lastSpaceConfig.writeEntry(m_connection->userId(), spaceId);
+    if (m_connection) {
+        m_lastSpaceConfig.writeEntry(m_connection->userId(), spaceId);
+    }
 
     if (!setRoom) {
         return;
