@@ -8,6 +8,7 @@
 #include <QObject>
 #include <QQmlEngine>
 #include <Quotient/room.h>
+#include <Quotient/roommember.h>
 #include <Quotient/uriresolver.h>
 
 #include "chatdocumenthandler.h"
@@ -21,6 +22,7 @@
 #include "models/sortfilterroomtreemodel.h"
 #include "models/sortfilterspacelistmodel.h"
 #include "models/timelinemodel.h"
+#include "models/userlistmodel.h"
 
 class NeoChatRoom;
 class NeoChatConnection;
@@ -119,6 +121,14 @@ class RoomManager : public QObject, public UriResolverBase
      */
     Q_PROPERTY(MediaMessageFilterModel *mediaMessageFilterModel READ mediaMessageFilterModel CONSTANT)
 
+    /**
+     * @brief The UserListModel that should be used for room member visualisation.
+     *
+     * @note Available here so that the room page and drawer both have access to the
+     *       same model.
+     */
+    Q_PROPERTY(UserListModel *userListModel READ userListModel CONSTANT)
+
     /**
      * @brief Whether a room is currently open in NeoChat.
      *
@@ -154,6 +164,9 @@ public:
     MessageFilterModel *messageFilterModel() const;
     MediaMessageFilterModel *mediaMessageFilterModel() const;
 
+    UserListModel *userListModel() const;
+    Q_INVOKABLE void activateUserModel();
+
     /**
      * @brief Resolve the given URI resource.
      *
@@ -203,7 +216,7 @@ public:
      */
     Q_INVOKABLE void maximizeMedia(int index);
 
-    Q_INVOKABLE void maximizeCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language);
+    Q_INVOKABLE void maximizeCode(const RoomMember &author, const QDateTime &time, const QString &codeText, const QString &language);
 
     /**
      * @brief Request that any full screen overlay currently open closes.
@@ -260,7 +273,7 @@ Q_SIGNALS:
      * Ask current room to open the user's details for the give user.
      * This assumes the user is loaded.
      */
-    void showUserDetail(const Quotient::User *user);
+    void showUserDetail(const Quotient::User *user, const NeoChatRoom *room);
 
     /**
      * @brief Request a media item is shown maximized.
@@ -274,7 +287,7 @@ Q_SIGNALS:
     /**
      * @brief Request a block of code is shown maximized.
      */
-    void showMaximizedCode(const QVariantMap &author, const QDateTime &time, const QString &codeText, const QString &language);
+    void showMaximizedCode(const RoomMember &author, const QDateTime &time, const QString &codeText, const QString &language);
 
     /**
      * @brief Request that any full screen overlay closes.
@@ -290,7 +303,7 @@ Q_SIGNALS:
      * @brief Request to show a menu for the given event.
      */
     void showMessageMenu(const QString &eventId,
-                         const QVariantMap &author,
+                         const Quotient::RoomMember &author,
                          MessageComponentType::Type messageComponentType,
                          const QString &plainText,
                          const QString &htmlText,
@@ -300,7 +313,7 @@ Q_SIGNALS:
      * @brief Request to show a menu for the given media event.
      */
     void showFileMenu(const QString &eventId,
-                      const QVariantMap &author,
+                      const Quotient::RoomMember &author,
                       MessageComponentType::Type messageComponentType,
                       const QString &plainText,
                       const QString &mimeType,
@@ -358,6 +371,9 @@ private:
     TimelineModel *m_timelineModel;
     MessageFilterModel *m_messageFilterModel;
     MediaMessageFilterModel *m_mediaMessageFilterModel;
+
+    UserListModel *m_userListModel;
+
     QPointer<NeoChatConnection> m_connection;
 
     void setCurrentRoom(const QString &roomId);
